from typing import Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..models.image_media_source_ref_type import ImageMediaSourceRefType
from ..types import UNSET, Unset

T = TypeVar("T", bound="ImageMediaSourceRef")


@_attrs_define
class ImageMediaSourceRef:
    """The media source of the image

    Attributes:
        uid (str): The unique identifier of the media source
        name (str): The name of the media source
        type (Union[Unset, ImageMediaSourceRefType]): The type of the media source
    """

    uid: str
    name: str
    type: Union[Unset, ImageMediaSourceRefType] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        uid = self.uid

        name = self.name

        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "uid": uid,
                "name": name,
            }
        )
        if type is not UNSET:
            field_dict["type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        uid = d.pop("uid")

        name = d.pop("name")

        _type = d.pop("type", UNSET)
        type: Union[Unset, ImageMediaSourceRefType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = ImageMediaSourceRefType(_type)

        image_media_source_ref = cls(
            uid=uid,
            name=name,
            type=type,
        )

        return image_media_source_ref



from impresso.api_client.client import AuthenticatedClient
from impresso.resources.collections import CollectionsResource
from impresso.resources.content_items import ContentItemsResource
from impresso.resources.data_providers import DataProvidersResource
from impresso.resources.entities import EntitiesResource
from impresso.resources.experiments import ExperimentsResource
from impresso.resources.images import ImagesResource
from impresso.resources.media_sources import MediaSourcesResource
from impresso.resources.search import SearchResource
from impresso.resources.text_reuse import TextReuseDomain
from impresso.resources.tools import ToolsResource
from impresso.resources.topics import TopicsResource


class ImpressoApiResourcesBase:
    """Base class for the Impresso client that contains resources."""

    def __init__(self, api_client: AuthenticatedClient) -> None:
        self._api_client = api_client

    @property
    def search(self) -> SearchResource:
        return SearchResource(self._api_client)

    @property
    def content_items(self) -> ContentItemsResource:
        return ContentItemsResource(self._api_client)

    @property
    def text_reuse(self) -> TextReuseDomain:
        return TextReuseDomain(self._api_client)

    @property
    def media_sources(self) -> MediaSourcesResource:
        return MediaSourcesResource(self._api_client)

    @property
    def collections(self) -> CollectionsResource:
        return CollectionsResource(self._api_client)

    @property
    def entities(self) -> EntitiesResource:
        return EntitiesResource(self._api_client)

    @property
    def images(self) -> ImagesResource:
        return ImagesResource(self._api_client)

    @property
    def tools(self) -> ToolsResource:
        return ToolsResource(self._api_client)

    @property
    def experiments(self) -> ExperimentsResource:
        return ExperimentsResource(self._api_client)

    @property
    def topics(self) -> TopicsResource:
        return TopicsResource(self._api_client)

    @property
    def data_providers(self) -> DataProvidersResource:
        return DataProvidersResource(self._api_client)



from typing import Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..types import UNSET, Unset

T = TypeVar("T", bound="TopicMention")


@_attrs_define
class TopicMention:
    """Topic presence in a content item.

    Attributes:
        uid (str): Unique identifier of the topic.
        relevance (Union[Unset, float]): Relevance of the topic in the content item.
    """

    uid: str
    relevance: Union[Unset, float] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        uid = self.uid

        relevance = self.relevance

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "uid": uid,
            }
        )
        if relevance is not UNSET:
            field_dict["relevance"] = relevance

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        uid = d.pop("uid")

        relevance = d.pop("relevance", UNSET)

        topic_mention = cls(
            uid=uid,
            relevance=relevance,
        )

        return topic_mention



from enum import Enum
from typing import Literal


class GetSearchFacetId(str, Enum):
    ACCESSRIGHT = "accessRight"
    COLLECTION = "collection"
    CONTENTLENGTH = "contentLength"
    COPYRIGHT = "copyright"
    COUNTRY = "country"
    DATADOMAIN = "dataDomain"
    DATERANGE = "daterange"
    LANGUAGE = "language"
    LOCATION = "location"
    MONTH = "month"
    NAG = "nag"
    NEWSPAPER = "newspaper"
    ORGANISATION = "organisation"
    PARTNER = "partner"
    PERSON = "person"
    SOURCEMEDIUM = "sourceMedium"
    SOURCETYPE = "sourceType"
    TOPIC = "topic"
    TYPE = "type"
    YEAR = "year"

    def __str__(self) -> str:
        return str(self.value)


GetSearchFacetIdLiteral = Literal[
    "accessRight",
    "collection",
    "contentLength",
    "copyright",
    "country",
    "dataDomain",
    "daterange",
    "language",
    "location",
    "month",
    "nag",
    "newspaper",
    "organisation",
    "partner",
    "person",
    "sourceMedium",
    "sourceType",
    "topic",
    "type",
    "year",
]



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="Error")


@_attrs_define
class Error:
    """Error response that follows https://datatracker.ietf.org/doc/html/rfc7807#section-3.1

    Attributes:
        type (str): A URI reference [RFC3986] that identifies the problem type.
        title (str): A short, human-readable summary of the problem type.
        status (int): The HTTP status code ([RFC7231], Section 6)
        detail (Union[Unset, str]): A human-readable explanation specific to this occurrence of the problem.
    """

    type: str
    title: str
    status: int
    detail: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        type = self.type

        title = self.title

        status = self.status

        detail = self.detail

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type,
                "title": title,
                "status": status,
            }
        )
        if detail is not UNSET:
            field_dict["detail"] = detail

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        type = d.pop("type")

        title = d.pop("title")

        status = d.pop("status")

        detail = d.pop("detail", UNSET)

        error = cls(
            type=type,
            title=title,
            status=status,
            detail=detail,
        )

        error.additional_properties = d
        return error

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar

from attrs import define as _attrs_define

T = TypeVar("T", bound="VersionDetails")


@_attrs_define
class VersionDetails:
    """Details of the current version of the API and details of its aspects.

    Attributes:
        version (str): Version of the API.
    """

    version: str

    def to_dict(self) -> Dict[str, Any]:
        version = self.version

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "version": version,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        version = d.pop("version")

        version_details = cls(
            version=version,
        )

        return version_details



from typing import Any, Callable, Iterator, cast

from pandas import DataFrame, json_normalize

from impresso.api_client.api.data_providers import (
    find_data_providers,
    get_data_provider,
)
from impresso.api_client.models.find_data_providers_base_find_response import (
    FindDataProvidersBaseFindResponse,
)
from impresso.api_client.types import UNSET
from impresso.api_models import BaseFind, Filter
from impresso.data_container import DataContainer, iterate_pages
from impresso.resources.base import Resource
from impresso.structures import AND, OR
from impresso.util.error import raise_for_error
from impresso.util.filters import and_or_filter


class FindDataProvidersSchema(BaseFind):
    """Schema for the find data providers response."""

    data: list[dict]


class FindDataProvidersContainer(DataContainer):
    """Response of a find call."""

    def __init__(
        self,
        data: FindDataProvidersBaseFindResponse,
        pydantic_model: type[FindDataProvidersSchema],
        fetch_method: Callable[..., "FindDataProvidersContainer"],
        fetch_method_args: dict[str, Any],
        web_app_search_result_url: str | None = None,
    ):
        super().__init__(data, pydantic_model, web_app_search_result_url)
        self._fetch_method = fetch_method
        self._fetch_method_args = fetch_method_args

    @property
    def df(self) -> DataFrame:
        """Return the data as a pandas dataframe."""
        data = self._data.to_dict()["data"]
        if len(data):
            return json_normalize(self._data.to_dict()["data"]).set_index("id")
        return DataFrame()

    def pages(self) -> Iterator["FindDataProvidersContainer"]:
        """Iterate over all pages of results."""
        yield self
        yield from iterate_pages(
            self._fetch_method,
            self._fetch_method_args,
            self.offset,
            self.limit,
            self.total,
        )


class GetDataProviderContainer(DataContainer):
    """Response of a get call."""

    @property
    def df(self) -> DataFrame:
        """Return the data as a pandas dataframe."""
        data = self._data.to_dict()
        if len(data):
            return json_normalize([self._data.to_dict()]).set_index("id")
        return DataFrame()


class DataProvidersResource(Resource):
    """Search data providers in the Impresso database.

    Data providers are partner institutions that provide content to Impresso,
    such as libraries, archives, and media organizations.

    Examples:
        Find all data providers:
        >>> results = data_providers.find()  # doctest: +SKIP
        >>> print(results.df)  # doctest: +SKIP

        Search data providers by name:
        >>> results = data_providers.find(term="library")  # doctest: +SKIP
        >>> print(results.df)  # doctest: +SKIP

        Get a specific data provider by its ID:
        >>> provider_id = "some-provider-id"  # Replace with a real ID
        >>> provider = data_providers.get(provider_id)  # doctest: +SKIP
        >>> print(provider.df)  # doctest: +SKIP
    """

    name = "data_providers"

    def find(
        self,
        term: str | None = None,
        provider_id: str | AND[str] | OR[str] | None = None,
        limit: int | None = None,
        offset: int | None = None,
    ) -> FindDataProvidersContainer:
        """
        Search data providers in Impresso.

        Data providers are partner institutions that provide content to Impresso,
        such as libraries, archives, and media organizations.

        Args:
            term: Search term to find data providers by their names.
            provider_id: Return only data provider with this ID.
            limit: Number of results to return.
            offset: Number of results to skip.

        Returns:
            FindDataProvidersContainer: Data container with a page of results of the search.
        """

        filters: list[Filter] = []
        if provider_id is not None:
            filters.extend(and_or_filter(provider_id, "id"))

        # filters_pb = filters_as_protobuf(filters or [])

        result = find_data_providers.sync(
            client=self._api_client,
            term=term if term is not None else UNSET,
            limit=limit if limit is not None else UNSET,
            offset=offset if offset is not None else UNSET,
        )
        raise_for_error(result)
        return FindDataProvidersContainer(
            cast(FindDataProvidersBaseFindResponse, result),
            FindDataProvidersSchema,
            fetch_method=self.find,
            fetch_method_args={
                "term": term,
                "provider_id": provider_id,
            },
            web_app_search_result_url=(
                _build_web_app_find_data_providers_url(
                    base_url=self._get_web_app_base_url(),
                    term=term,
                )
                if provider_id is None
                else None
            ),
        )

    def get(self, id: str) -> GetDataProviderContainer:
        """Get data provider by ID.

        Args:
            id: The ID of the data provider to retrieve.

        Returns:
            GetDataProviderContainer: Data container with the data provider information.
        """

        result = get_data_provider.sync(
            client=self._api_client,
            id=id,
        )
        raise_for_error(result)
        return GetDataProviderContainer(
            result,
            FindDataProvidersSchema,
            web_app_search_result_url=_build_web_app_get_data_provider_url(
                base_url=self._get_web_app_base_url(),
                id=id,
            ),
        )


def _build_web_app_find_data_providers_url(
    base_url: str,
    term: str | None = None,
) -> str:
    query_params = {
        "q": term,
    }
    query_string = "&".join(
        f"{key}={value}" for key, value in query_params.items() if value is not None
    )
    url = f"{base_url}/data-providers"
    return f"{url}?{query_string}" if query_string else url


def _build_web_app_get_data_provider_url(
    base_url: str,
    id: str,
) -> str:
    return f"{base_url}/data-providers/{id}"



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.image_image_types import ImageImageTypes
    from ..models.image_media_source_ref import ImageMediaSourceRef


T = TypeVar("T", bound="Image")


@_attrs_define
class Image:
    """An image from a content item

    Attributes:
        uid (str): The unique identifier of the image
        issue_uid (str): The unique identifier of the issue that the image belongs to.
        preview_url (str): The URL of the image preview
        media_source_ref (ImageMediaSourceRef): The media source of the image
        date (datetime.date): The date of the image or the date of the issue that the image belongs to.
        caption (Union[Unset, str]): Image caption
        content_item_uid (Union[Unset, str]): The unique identifier of the content item that the image belongs to.
        page_numbers (Union[Unset, List[int]]): The page numbers of the issue that the image belongs to.
        image_types (Union[Unset, ImageImageTypes]):
        embeddings (Union[Unset, List[str]]): Precomputed embeddings for the image in the format:
            <model_type>:<base64_embedding_vector>.
    """

    uid: str
    issue_uid: str
    preview_url: str
    media_source_ref: "ImageMediaSourceRef"
    date: datetime.date
    caption: Union[Unset, str] = UNSET
    content_item_uid: Union[Unset, str] = UNSET
    page_numbers: Union[Unset, List[int]] = UNSET
    image_types: Union[Unset, "ImageImageTypes"] = UNSET
    embeddings: Union[Unset, List[str]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        uid = self.uid

        issue_uid = self.issue_uid

        preview_url = self.preview_url

        media_source_ref = self.media_source_ref.to_dict()

        date = self.date.isoformat()

        caption = self.caption

        content_item_uid = self.content_item_uid

        page_numbers: Union[Unset, List[int]] = UNSET
        if not isinstance(self.page_numbers, Unset):
            page_numbers = self.page_numbers

        image_types: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.image_types, Unset):
            image_types = self.image_types.to_dict()

        embeddings: Union[Unset, List[str]] = UNSET
        if not isinstance(self.embeddings, Unset):
            embeddings = self.embeddings

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "uid": uid,
                "issueUid": issue_uid,
                "previewUrl": preview_url,
                "mediaSourceRef": media_source_ref,
                "date": date,
            }
        )
        if caption is not UNSET:
            field_dict["caption"] = caption
        if content_item_uid is not UNSET:
            field_dict["contentItemUid"] = content_item_uid
        if page_numbers is not UNSET:
            field_dict["pageNumbers"] = page_numbers
        if image_types is not UNSET:
            field_dict["imageTypes"] = image_types
        if embeddings is not UNSET:
            field_dict["embeddings"] = embeddings

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.image_image_types import ImageImageTypes
        from ..models.image_media_source_ref import ImageMediaSourceRef

        d = src_dict.copy()
        uid = d.pop("uid")

        issue_uid = d.pop("issueUid")

        preview_url = d.pop("previewUrl")

        media_source_ref = ImageMediaSourceRef.from_dict(d.pop("mediaSourceRef"))

        date = isoparse(d.pop("date")).date()

        caption = d.pop("caption", UNSET)

        content_item_uid = d.pop("contentItemUid", UNSET)

        page_numbers = cast(List[int], d.pop("pageNumbers", UNSET))

        _image_types = d.pop("imageTypes", UNSET)
        image_types: Union[Unset, ImageImageTypes]
        if isinstance(_image_types, Unset):
            image_types = UNSET
        else:
            image_types = ImageImageTypes.from_dict(_image_types)

        embeddings = cast(List[str], d.pop("embeddings", UNSET))

        image = cls(
            uid=uid,
            issue_uid=issue_uid,
            preview_url=preview_url,
            media_source_ref=media_source_ref,
            date=date,
            caption=caption,
            content_item_uid=content_item_uid,
            page_numbers=page_numbers,
            image_types=image_types,
            embeddings=embeddings,
        )

        return image



from enum import Enum
from typing import Literal


class FindTextReuseClustersOrderBy(str, Enum):
    PASSAGES_COUNT = "passages-count"
    VALUE_1 = "-passages-count"

    def __str__(self) -> str:
        return str(self.value)


FindTextReuseClustersOrderByLiteral = Literal[
    "passages-count",
    "-passages-count",
]



from typing import Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..types import UNSET, Unset

T = TypeVar("T", bound="RemoveCollectionResponseTask")


@_attrs_define
class RemoveCollectionResponseTask:
    """Deletion task details

    Attributes:
        task_id (Union[Unset, str]): The ID of the task
        creation_date (Union[Unset, str]): When task was created
    """

    task_id: Union[Unset, str] = UNSET
    creation_date: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        task_id = self.task_id

        creation_date = self.creation_date

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if task_id is not UNSET:
            field_dict["task_id"] = task_id
        if creation_date is not UNSET:
            field_dict["creationDate"] = creation_date

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        task_id = d.pop("task_id", UNSET)

        creation_date = d.pop("creationDate", UNSET)

        remove_collection_response_task = cls(
            task_id=task_id,
            creation_date=creation_date,
        )

        return remove_collection_response_task



from typing import Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..types import UNSET, Unset

T = TypeVar("T", bound="TopicWord")


@_attrs_define
class TopicWord:
    """A word included in a topic

    Attributes:
        w (str): Word surface form
        p (float): Probability of the word in topic
        h (Union[Unset, bool]): If word is highlighted
    """

    w: str
    p: float
    h: Union[Unset, bool] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        w = self.w

        p = self.p

        h = self.h

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "w": w,
                "p": p,
            }
        )
        if h is not UNSET:
            field_dict["h"] = h

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        w = d.pop("w")

        p = d.pop("p")

        h = d.pop("h", UNSET)

        topic_word = cls(
            w=w,
            p=p,
            h=h,
        )

        return topic_word



from typing import TYPE_CHECKING, Any, Dict, Type, TypeVar

from attrs import define as _attrs_define

if TYPE_CHECKING:
    from ..models.remove_collection_response_params import RemoveCollectionResponseParams
    from ..models.remove_collection_response_task import RemoveCollectionResponseTask


T = TypeVar("T", bound="RemoveCollectionResponse")


@_attrs_define
class RemoveCollectionResponse:
    """Remove collection response

    Attributes:
        params (RemoveCollectionResponseParams):
        task (RemoveCollectionResponseTask): Deletion task details
    """

    params: "RemoveCollectionResponseParams"
    task: "RemoveCollectionResponseTask"

    def to_dict(self) -> Dict[str, Any]:
        params = self.params.to_dict()

        task = self.task.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "params": params,
                "task": task,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.remove_collection_response_params import RemoveCollectionResponseParams
        from ..models.remove_collection_response_task import RemoveCollectionResponseTask

        d = src_dict.copy()
        params = RemoveCollectionResponseParams.from_dict(d.pop("params"))

        task = RemoveCollectionResponseTask.from_dict(d.pop("task"))

        remove_collection_response = cls(
            params=params,
            task=task,
        )

        return remove_collection_response



from enum import Enum
from typing import Literal


class ImpressoNamedEntityRecognitionRequestMethod(str, Enum):
    NEL = "nel"
    NER = "ner"
    NER_NEL = "ner-nel"

    def __str__(self) -> str:
        return str(self.value)


ImpressoNamedEntityRecognitionRequestMethodLiteral = Literal[
    "nel",
    "ner",
    "ner-nel",
]



from typing import Any, Dict, Type, TypeVar

from attrs import define as _attrs_define

T = TypeVar("T", bound="FacetWithLabel")


@_attrs_define
class FacetWithLabel:
    """An facet that has a value and a label

    Attributes:
        id (str): Unique identifier of the facet
        label (str): Label of the facet
    """

    id: str
    label: str

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        label = self.label

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "id": id,
                "label": label,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        label = d.pop("label")

        facet_with_label = cls(
            id=id,
            label=label,
        )

        return facet_with_label



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define

from ..models.add_collectable_items_from_filters_namespace import AddCollectableItemsFromFiltersNamespace

if TYPE_CHECKING:
    from ..models.filter_ import Filter


T = TypeVar("T", bound="AddCollectableItemsFromFilters")


@_attrs_define
class AddCollectableItemsFromFilters:
    """Request to add content items to a collection from content items that match given filters

    Attributes:
        filters (List['Filter']): Filters to apply when selecting items to add to the collection
        namespace (AddCollectableItemsFromFiltersNamespace): Namespace to use when selecting items to add to the
            collection
    """

    filters: List["Filter"]
    namespace: AddCollectableItemsFromFiltersNamespace

    def to_dict(self) -> Dict[str, Any]:
        filters = []
        for filters_item_data in self.filters:
            filters_item = filters_item_data.to_dict()
            filters.append(filters_item)

        namespace = self.namespace.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "filters": filters,
                "namespace": namespace,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.filter_ import Filter

        d = src_dict.copy()
        filters = []
        _filters = d.pop("filters")
        for filters_item_data in _filters:
            filters_item = Filter.from_dict(filters_item_data)

            filters.append(filters_item)

        namespace = AddCollectableItemsFromFiltersNamespace(d.pop("namespace"))

        add_collectable_items_from_filters = cls(
            filters=filters,
            namespace=namespace,
        )

        return add_collectable_items_from_filters



from typing import Any, Dict, Type, TypeVar

from attrs import define as _attrs_define

T = TypeVar("T", bound="AuthenticationResponseUser")


@_attrs_define
class AuthenticationResponseUser:
    """User details

    Attributes:
        id (int):
        username (str):
        firstname (str):
        lastname (str):
        is_staff (bool):
        is_active (bool):
        is_superuser (bool):
        uid (str):
    """

    id: int
    username: str
    firstname: str
    lastname: str
    is_staff: bool
    is_active: bool
    is_superuser: bool
    uid: str

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        username = self.username

        firstname = self.firstname

        lastname = self.lastname

        is_staff = self.is_staff

        is_active = self.is_active

        is_superuser = self.is_superuser

        uid = self.uid

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "id": id,
                "username": username,
                "firstname": firstname,
                "lastname": lastname,
                "isStaff": is_staff,
                "isActive": is_active,
                "isSuperuser": is_superuser,
                "uid": uid,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        username = d.pop("username")

        firstname = d.pop("firstname")

        lastname = d.pop("lastname")

        is_staff = d.pop("isStaff")

        is_active = d.pop("isActive")

        is_superuser = d.pop("isSuperuser")

        uid = d.pop("uid")

        authentication_response_user = cls(
            id=id,
            username=username,
            firstname=firstname,
            lastname=lastname,
            is_staff=is_staff,
            is_active=is_active,
            is_superuser=is_superuser,
            uid=uid,
        )

        return authentication_response_user



import matplotlib.pyplot as plt
import io
import base64
from typing import Any, Callable, Iterator, cast

from pandas import DataFrame, json_normalize

from impresso.api_client.api.search import search
from impresso.api_client.api.search_facets import get_search_facet
from impresso.api_client.models.content_item_copyright_status import (
    ContentItemCopyrightStatusLiteral,
)
from impresso.api_client.models.get_search_facet_id import (
    GetSearchFacetId,
    GetSearchFacetIdLiteral,
)
from impresso.api_client.models.get_search_facet_order_by import (
    GetSearchFacetOrderBy,
    GetSearchFacetOrderByLiteral,
)
from impresso.api_client.models.get_search_facet_base_find_response import (
    GetSearchFacetBaseFindResponse,
)
from impresso.api_client.models.search_order_by import (
    SearchOrderBy,
    SearchOrderByLiteral,
)
from impresso.api_client.models.search_base_find_response import (
    SearchBaseFindResponse,
)
from impresso.api_client.types import UNSET, Unset
from impresso.api_models import ContentItem, BaseFind, Filter, Q, SearchFacetBucket
from impresso.data_container import DataContainer, iterate_pages
from impresso.resources.base import DEFAULT_PAGE_SIZE, Resource
from impresso.resources.tools import Embedding
from impresso.structures import AND, OR, DateRange
from impresso.util.error import raise_for_error
from impresso.util.filters import and_or_filter, filters_as_protobuf
from impresso.util.py import get_enum_from_literal


class SearchResponseSchema(BaseFind):
    """Schema for the content items response."""

    data: list[ContentItem]


class FacetResponseSchema(BaseFind):
    """Schema for a facet response."""

    data: list[SearchFacetBucket]


class SearchDataContainer(DataContainer):
    """Response of a search call."""

    def __init__(
        self,
        data: SearchBaseFindResponse,
        pydantic_model: type[SearchResponseSchema],
        fetch_method: Callable[..., "SearchDataContainer"],
        fetch_method_args: dict[str, Any],
        web_app_search_result_url: str | None = None,
    ):
        super().__init__(data, pydantic_model, web_app_search_result_url)
        self._fetch_method = fetch_method
        self._fetch_method_args = fetch_method_args

    @property
    def df(self) -> DataFrame:
        """Return the data as a pandas dataframe."""
        data = self._data.to_dict()["data"]
        if len(data):
            return json_normalize(self._data.to_dict()["data"]).set_index("uid")
        return DataFrame()

    def pages(self) -> Iterator["SearchDataContainer"]:
        """Iterate over all pages of results."""
        yield self
        yield from iterate_pages(
            self._fetch_method,
            self._fetch_method_args,
            self.offset,
            self.limit,
            self.total,
        )


class FacetDataContainer(DataContainer):
    """Response of a get facet call."""

    def __init__(
        self,
        data: GetSearchFacetBaseFindResponse,
        pydantic_model: type[FacetResponseSchema],
        fetch_method: Callable[..., "FacetDataContainer"],
        fetch_method_args: dict[str, Any],
        web_app_search_result_url: str | None = None,
    ):
        super().__init__(data, pydantic_model, web_app_search_result_url)
        self._fetch_method = fetch_method
        self._fetch_method_args = fetch_method_args

    @property
    def df(self) -> DataFrame:
        """Return the data as a pandas dataframe."""
        data = self._data.to_dict()["data"]
        if len(data):
            return json_normalize(self._data.to_dict()["data"]).set_index("value")
        return DataFrame()

    def _get_preview_image_(self) -> str | None:
        if self.size == 0:
            return None
        return render_dataframe_chart_base64(
            self.df.index[:].values, self.df["count"].values
        )

    def pages(self) -> Iterator["FacetDataContainer"]:
        """Iterate over all pages of results."""
        yield self
        yield from iterate_pages(
            self._fetch_method,
            self._fetch_method_args,
            self.offset,
            self.limit,
            self.total,
        )


class SearchResource(Resource):
    """Search content items in the impresso database.

    Examples:
        Search for articles containing a term:
        >>> results = search.find(term="war")  # doctest: +SKIP
        >>> print(results.df)  # doctest: +SKIP

        Filter articles by date range and newspaper:
        >>> from impresso import DateRange
        >>> date_range = DateRange(start="1900-01-01", end="1910-12-31")
        >>> results = search.find(term="revolution", newspaper_id="GDL", date_range=date_range)  # doctest: +SKIP
        >>> print(results.df)  # doctest: +SKIP

        Search for front page articles mentioning an entity:
        >>> results = search.find(entity_id="aida-0001-54-Napoleon", front_page=True)  # doctest: +SKIP
        >>> print(results.df)  # doctest: +SKIP

        Search by semantic similarity using text embeddings:
        >>> embedding = tools.embed_text("military conflict", target="text")  # doctest: +SKIP
        >>> similar_articles = search.find(embedding=embedding)  # doctest: +SKIP
        >>> print(similar_articles.df)  # doctest: +SKIP

        Get facets to analyze search results:
        >>> newspaper_facets = search.facet(facet="newspaper", term="war")  # doctest: +SKIP
        >>> print(newspaper_facets.df)  # doctest: +SKIP
    """

    name = "search"

    def find(
        self,
        term: str | AND[str] | OR[str] | None = None,
        order_by: SearchOrderByLiteral | None = None,
        limit: int | None = None,
        offset: int | None = None,
        with_text_contents: bool | None = False,
        title: str | AND[str] | OR[str] | None = None,
        front_page: bool | None = None,
        entity_id: str | AND[str] | OR[str] | None = None,
        newspaper_id: str | AND[str] | OR[str] | None = None,
        date_range: DateRange | None = None,
        language: str | AND[str] | OR[str] | None = None,
        mention: str | AND[str] | OR[str] | None = None,
        topic_id: str | AND[str] | OR[str] | None = None,
        collection_id: str | AND[str] | OR[str] | None = None,
        country: str | AND[str] | OR[str] | None = None,
        partner_id: str | AND[str] | OR[str] | None = None,
        issue_id: str | OR[str] | None = None,
        text_reuse_cluster_id: str | AND[str] | OR[str] | None = None,
        embedding: Embedding | None = None,
        copyright: (
            ContentItemCopyrightStatusLiteral
            | AND[ContentItemCopyrightStatusLiteral]
            | OR[ContentItemCopyrightStatusLiteral]
            | None
        ) = None,
        include_embeddings: bool = False,
    ) -> SearchDataContainer:
        """Search for content items in Impresso.

        Args:
            term: Search term or combination of search terms.
            order_by: Sort order for results.
            limit: Maximum number of results to return per page. Defaults to 100.
            offset: Number of results to skip for pagination.
            with_text_contents: Return only content items with text contents. Defaults to False.
            title: Filter by content items having this term or all/any of the terms in the title.
            front_page: Return only content items that were on the front page.
            entity_id: Filter by content items mentioning this entity or all/any of the entities.
            newspaper_id: Filter by newspaper ID(s).
            date_range: Filter by publication date range.
            language: Filter by content language or all/any of the languages.
                Use 2-letter ISO language codes (e.g., 'en', 'de', 'fr').
            mention: Filter by content items mentioning entities with this term or all/any of
                entities with the terms.
            topic_id: Filter by topic ID(s).
            collection_id: Filter by collection ID(s).
            country: Filter by country of publication. Use 2-letter ISO country codes
                (e.g., 'ch', 'de', 'lu').
            partner_id: Filter by partner institution ID(s).
            text_reuse_cluster_id: Filter by text reuse cluster ID(s).
            embedding: Text embedding for similarity search. Use `tools.embed_text()` to generate
                embeddings from text.
            copyright: Filter by copyright status.
            include_embeddings: Whether to include embeddings in the response. Defaults to False.

        Returns:
            SearchDataContainer: Data container with the first page of search results.
        """

        page_limit = limit if limit is not None else DEFAULT_PAGE_SIZE
        embedding_with_limit = f"{embedding}:{page_limit}" if embedding else None

        filters = self._build_filters(
            string=term,
            with_text_contents=with_text_contents,
            title=title,
            front_page=front_page,
            entity_id=entity_id,
            newspaper_id=newspaper_id,
            date_range=date_range,
            language=language,
            mention=mention,
            topic_id=topic_id,
            collection_id=collection_id,
            country=country,
            partner_id=partner_id,
            text_reuse_cluster_id=text_reuse_cluster_id,
            embedding=embedding_with_limit,
            copyright=copyright,
            issue_id=issue_id,
        )

        filters_pb = filters_as_protobuf(filters or [])

        result = search.sync(
            client=self._api_client,
            term=UNSET,
            order_by=(
                get_enum_from_literal(order_by, SearchOrderBy)
                if order_by is not None
                else UNSET
            ),
            filters=filters_pb if filters_pb else UNSET,
            limit=page_limit,
            offset=offset if offset is not None else UNSET,
            include_embeddings=include_embeddings if include_embeddings else UNSET,
        )
        raise_for_error(result)
        return SearchDataContainer(
            cast(SearchBaseFindResponse, result),
            SearchResponseSchema,
            fetch_method=self.find,
            fetch_method_args={
                "term": term,
                "order_by": order_by,
                "with_text_contents": with_text_contents,
                "title": title,
                "front_page": front_page,
                "entity_id": entity_id,
                "newspaper_id": newspaper_id,
                "date_range": date_range,
                "language": language,
                "mention": mention,
                "topic_id": topic_id,
                "collection_id": collection_id,
                "country": country,
                "partner_id": partner_id,
                "text_reuse_cluster_id": text_reuse_cluster_id,
                "embedding": embedding_with_limit,
            },
            web_app_search_result_url=_build_web_app_search_url(
                f"{self._get_web_app_base_url()}/search",
                order_by=order_by,
                filters=filters_pb,
                limit=limit,
                offset=offset,
            ),
        )

    def facet(
        self,
        facet: GetSearchFacetIdLiteral,
        term: str | AND[str] | OR[str] | None = None,
        order_by: GetSearchFacetOrderByLiteral | None = "value",
        limit: int | None = None,
        offset: int | None = None,
        with_text_contents: bool | None = False,
        title: str | AND[str] | OR[str] | None = None,
        front_page: bool | None = None,
        entity_id: str | AND[str] | OR[str] | None = None,
        newspaper_id: str | AND[str] | OR[str] | None = None,
        date_range: DateRange | None = None,
        language: str | AND[str] | OR[str] | None = None,
        mention: str | AND[str] | OR[str] | None = None,
        topic_id: str | AND[str] | OR[str] | None = None,
        collection_id: str | AND[str] | OR[str] | None = None,
        country: str | AND[str] | OR[str] | None = None,
        partner_id: str | AND[str] | OR[str] | None = None,
        text_reuse_cluster_id: str | AND[str] | OR[str] | None = None,
    ) -> FacetDataContainer:
        """Get facets for a search query.

        Facets provide aggregated information about a specific dimension of search results,
        such as counts of newspaper titles, languages, or topics.

        Args:
            facet: Type of facet to retrieve (e.g., 'newspaper', 'language', 'topic').
            term: Search term or combination of terms to filter facets.
            order_by: Sort order for facet results ('value' or 'count'). Defaults to 'value'.
            limit: Maximum number of facet buckets to return.
            offset: Number of facet buckets to skip for pagination.
            with_text_contents: Filter for content items with text contents. Defaults to False.
            title: Filter by content items having this term or terms in the title.
            front_page: Filter for content items that were on the front page.
            entity_id: Filter by content items mentioning this entity or entities.
            newspaper_id: Filter by newspaper ID(s).
            date_range: Filter by publication date range.
            language: Filter by content language. Use 2-letter ISO language codes
                (e.g., 'en', 'de', 'fr').
            mention: Filter by content items mentioning entities with these terms.
            topic_id: Filter by topic ID(s).
            collection_id: Filter by collection ID(s).
            country: Filter by country of publication. Use 2-letter ISO country codes
                (e.g., 'ch', 'de', 'lu').
            partner_id: Filter by partner institution ID(s).
            text_reuse_cluster_id: Filter by text reuse cluster ID(s).

        Returns:
            FacetDataContainer: Data container with facet results, including counts for each bucket.

        Examples:
            >>> search = SearchResource(client)
            >>> # Get newspaper facets for articles mentioning "war"
            >>> newspaper_facets = search.facet(facet="newspaper", term="war")
            >>> # Get language facets for front page articles
            >>> language_facets = search.facet(facet="language", front_page=True)
        """

        facet_id = get_enum_from_literal(facet, GetSearchFacetId)
        if isinstance(facet_id, Unset):
            raise ValueError(f"{facet} is not a valid value")

        filters = self._build_filters(
            string=term,
            with_text_contents=with_text_contents,
            title=title,
            front_page=front_page,
            entity_id=entity_id,
            newspaper_id=newspaper_id,
            date_range=date_range,
            language=language,
            mention=mention,
            topic_id=topic_id,
            collection_id=collection_id,
            country=country,
            partner_id=partner_id,
            text_reuse_cluster_id=text_reuse_cluster_id,
        )

        filters_pb = filters_as_protobuf(filters or [])

        result = get_search_facet.sync(
            client=self._api_client,
            id=facet_id,
            filters=filters_pb if filters_pb else UNSET,
            offset=offset if offset is not None else UNSET,
            limit=limit if limit is not None else UNSET,
            order_by=(
                get_enum_from_literal(order_by, GetSearchFacetOrderBy)
                if order_by is not None
                else UNSET
            ),
        )
        raise_for_error(result)
        return FacetDataContainer(
            cast(GetSearchFacetBaseFindResponse, result),
            FacetResponseSchema,
            fetch_method=self.facet,
            fetch_method_args={
                "facet": facet,
                "term": term,
                "order_by": order_by,
                "with_text_contents": with_text_contents,
                "title": title,
                "front_page": front_page,
                "entity_id": entity_id,
                "newspaper_id": newspaper_id,
                "date_range": date_range,
                "language": language,
                "mention": mention,
                "topic_id": topic_id,
                "collection_id": collection_id,
                "country": country,
                "partner_id": partner_id,
                "text_reuse_cluster_id": text_reuse_cluster_id,
            },
            web_app_search_result_url=_build_web_app_facet_url(
                f"{self._get_web_app_base_url()}/search",
                facet=facet,
                filters=filters_pb,
                limit=limit,
                offset=offset,
            ),
        )
        # return FacetDataContainer(
        #     result,
        #     SearchFacet,
        #     limit=limit,
        #     offset=offset,
        #     web_app_search_result_url=_build_web_app_facet_url(
        #         f"{self._get_web_app_base_url()}/search",
        #         facet=facet,
        #         filters=filters_pb,
        #         limit=limit,
        #         offset=offset,
        #     ),
        # )

    def _build_filters(
        self,
        string: str | AND[str] | OR[str] | None,
        with_text_contents: bool | None = False,
        title: str | AND[str] | OR[str] | None = None,
        front_page: bool | None = None,
        entity_id: str | AND[str] | OR[str] | None = None,
        newspaper_id: str | AND[str] | OR[str] | None = None,
        date_range: DateRange | None = None,
        language: str | AND[str] | OR[str] | None = None,
        mention: str | AND[str] | OR[str] | None = None,
        topic_id: str | AND[str] | OR[str] | None = None,
        collection_id: str | AND[str] | OR[str] | None = None,
        country: str | AND[str] | OR[str] | None = None,
        partner_id: str | AND[str] | OR[str] | None = None,
        issue_id: str | OR[str] | None = None,
        text_reuse_cluster_id: str | AND[str] | OR[str] | None = None,
        embedding: Embedding | None = None,
        copyright: (
            ContentItemCopyrightStatusLiteral
            | AND[ContentItemCopyrightStatusLiteral]
            | OR[ContentItemCopyrightStatusLiteral]
            | None
        ) = None,
    ) -> list[Filter]:
        filters: list[Filter] = []
        if string:
            filters.extend(and_or_filter(string, "string"))
        if with_text_contents:
            filters.append(Filter(type="has_text_contents", daterange=None))
        if title is not None:
            filters.extend(and_or_filter(title, "title"))
        if front_page:
            filters.append(Filter(type="is_front", daterange=None))
        if entity_id is not None:
            filters.extend(and_or_filter(entity_id, "entity"))
        if newspaper_id is not None:
            filters.extend(and_or_filter(newspaper_id, "newspaper"))
        if date_range is not None:
            filters.append(
                Filter(
                    type="daterange",
                    q=Q(DateRange._as_filter_value(date_range)),
                    context="exclude" if date_range.inverted else "include",
                    daterange=None,
                )
            )
        if language is not None:
            filters.extend(and_or_filter(language, "language"))
        if mention is not None:
            filters.extend(and_or_filter(mention, "mention"))
        if topic_id is not None:
            filters.extend(and_or_filter(topic_id, "topic"))
        if collection_id is not None:
            filters.extend(and_or_filter(collection_id, "collection"))
        if country is not None:
            filters.extend(and_or_filter(country, "country"))
        if partner_id is not None:
            filters.extend(and_or_filter(partner_id, "partner"))
        if text_reuse_cluster_id is not None:
            filters.extend(and_or_filter(text_reuse_cluster_id, "text_reuse_cluster"))
        if embedding is not None:
            filters.extend(and_or_filter(embedding, "embedding"))
        if copyright is not None:
            filters.extend(and_or_filter(str(copyright), "copyright"))
        if issue_id is not None:
            filters.extend(and_or_filter(issue_id, "issue"))

        return filters


def _build_web_app_search_url(
    base_url: str,
    order_by: SearchOrderByLiteral | None = None,
    filters: str | None = None,
    limit: int | None = None,
    offset: int | None = None,
) -> str:
    page = offset // limit if limit is not None and offset is not None else 0
    query_params = {
        "orderBy": order_by,
        "sq": filters,
        "p": page + 1,
    }
    query_string = "&".join(
        f"{key}={value}" for key, value in query_params.items() if value is not None
    )
    return f"{base_url}?{query_string}" if query_string else base_url


def _build_web_app_facet_url(
    base_url: str,
    facet: str,
    filters: str | None = None,
    limit: int | None = None,
    offset: int | None = None,
) -> str:
    page = offset // limit if limit is not None and offset is not None else 0
    query_params = {
        "index": "search",
        "facet": "type",
        "domain": facet,
        "sq": filters,
        "p": page + 1,
    }
    query_string = "&".join(
        f"{key}={value}" for key, value in query_params.items() if value is not None
    )
    return f"{base_url}?{query_string}" if query_string else base_url


def render_dataframe_chart_base64(x, y) -> str:
    plt.figure(figsize=(12, 1))

    if len(x) < 50:
        plt.bar(x, y)
    else:
        plt.plot(x, y)

    # Remove labels, and legend
    # plt.legend().remove()

    # Either
    # Remove axes,
    plt.axis("off")

    # OR
    # Remove Y axis and only show the first and last tick
    # plt.gca().get_yaxis().set_visible(False)
    # plt.xticks([x[0], x[-1]])

    # Remove the box around the graph
    for spine in plt.gca().spines.values():
        spine.set_visible(False)

    # Save the plot to a bytes buffer with a transparent background
    buffer = io.BytesIO()
    plt.savefig(
        buffer, format="png", bbox_inches="tight", pad_inches=0.1, transparent=True
    )
    buffer.seek(0)

    # Encode the bytes as base64
    image_base64 = base64.b64encode(buffer.getvalue()).decode("utf-8")

    # Close the plot to free up memory
    plt.close()

    return image_base64



from typing import TYPE_CHECKING, Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.text_reuse_cluster_time_coverage import TextReuseClusterTimeCoverage


T = TypeVar("T", bound="TextReuseCluster")


@_attrs_define
class TextReuseCluster:
    """Text reuse cluster details.

    Attributes:
        uid (str): Unique ID of the text reuse cluster.
        lexical_overlap (Union[Unset, float]): Overlap in percents between the passages in the cluster.
        cluster_size (Union[Unset, int]): Number of passages in the cluster.
        text_sample (Union[Unset, str]): Sample of a text from one of the passages in the cluster.
        time_coverage (Union[Unset, TextReuseClusterTimeCoverage]): Time coverage of the cluster.
    """

    uid: str
    lexical_overlap: Union[Unset, float] = UNSET
    cluster_size: Union[Unset, int] = UNSET
    text_sample: Union[Unset, str] = UNSET
    time_coverage: Union[Unset, "TextReuseClusterTimeCoverage"] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        uid = self.uid

        lexical_overlap = self.lexical_overlap

        cluster_size = self.cluster_size

        text_sample = self.text_sample

        time_coverage: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.time_coverage, Unset):
            time_coverage = self.time_coverage.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "uid": uid,
            }
        )
        if lexical_overlap is not UNSET:
            field_dict["lexicalOverlap"] = lexical_overlap
        if cluster_size is not UNSET:
            field_dict["clusterSize"] = cluster_size
        if text_sample is not UNSET:
            field_dict["textSample"] = text_sample
        if time_coverage is not UNSET:
            field_dict["timeCoverage"] = time_coverage

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.text_reuse_cluster_time_coverage import TextReuseClusterTimeCoverage

        d = src_dict.copy()
        uid = d.pop("uid")

        lexical_overlap = d.pop("lexicalOverlap", UNSET)

        cluster_size = d.pop("clusterSize", UNSET)

        text_sample = d.pop("textSample", UNSET)

        _time_coverage = d.pop("timeCoverage", UNSET)
        time_coverage: Union[Unset, TextReuseClusterTimeCoverage]
        if isinstance(_time_coverage, Unset):
            time_coverage = UNSET
        else:
            time_coverage = TextReuseClusterTimeCoverage.from_dict(_time_coverage)

        text_reuse_cluster = cls(
            uid=uid,
            lexical_overlap=lexical_overlap,
            cluster_size=cluster_size,
            text_sample=text_sample,
            time_coverage=time_coverage,
        )

        return text_reuse_cluster



from typing import Any, Callable, Iterator, cast

from pandas import DataFrame, json_normalize

from impresso.api_client.api.topics import find_topics, get_topic
from impresso.api_client.models.find_topics_base_find_response import (
    FindTopicsBaseFindResponse,
)
from impresso.api_client.models.find_topics_order_by import (
    FindTopicsOrderBy,
    FindTopicsOrderByLiteral,
)
from impresso.api_client.types import UNSET
from impresso.api_models import BaseFind
from impresso.data_container import DataContainer, iterate_pages
from impresso.resources.base import Resource
from impresso.util.error import raise_for_error
from impresso.util.py import get_enum_from_literal


class FindTopicsSchema(BaseFind):
    """Schema for the find topics response."""

    data: list[dict]


class FindTopicsContainer(DataContainer):
    """Response of a find call."""

    def __init__(
        self,
        data: FindTopicsBaseFindResponse,
        pydantic_model: type[FindTopicsSchema],
        fetch_method: Callable[..., "FindTopicsContainer"],
        fetch_method_args: dict[str, Any],
        web_app_search_result_url: str | None = None,
    ):
        super().__init__(data, pydantic_model, web_app_search_result_url)
        self._fetch_method = fetch_method
        self._fetch_method_args = fetch_method_args

    @property
    def df(self) -> DataFrame:
        """Return the data as a pandas dataframe."""
        data = self._data.to_dict()["data"]
        if len(data):
            return json_normalize(self._data.to_dict()["data"]).set_index("uid")
        return DataFrame()

    def pages(self) -> Iterator["FindTopicsContainer"]:
        """Iterate over all pages of results."""
        yield self
        yield from iterate_pages(
            self._fetch_method,
            self._fetch_method_args,
            self.offset,
            self.limit,
            self.total,
        )


class GetTopicContainer(DataContainer):
    """Response of a get call."""

    @property
    def df(self) -> DataFrame:
        """Return the data as a pandas dataframe."""
        data = self._data.to_dict()
        if len(data):
            return json_normalize([self._data.to_dict()]).set_index("uid")
        return DataFrame()


class TopicsResource(Resource):
    """
    Search topics in the Impresso database.

    Examples:
        Search for topics containing specific words:
        >>> results = topics.find(term="economy") # doctest: +SKIP
        >>> print(results.df) # doctest: +SKIP

        Get a specific topic by its ID:
        >>> topic_id = "some-topic-id" # Replace with a real ID
        >>> topic = topics.get(topic_id) # doctest: +SKIP
        >>> print(topic.df) # doctest: +SKIP

        Iterate through all pages of topic search results:
        >>> results = topics.find(term="war", limit=10) # doctest: +SKIP
        >>> for page in results.pages(): # doctest: +SKIP
        ...     print(page.df) # doctest: +SKIP
    """

    name = "topics"

    def find(
        self,
        term: str | None = None,
        order_by: FindTopicsOrderByLiteral | None = None,
        limit: int | None = None,
        offset: int | None = None,
    ) -> FindTopicsContainer:
        """
        Search topics in Impresso.

        Args:
            term: Search term to find topics by their words.
            order_by: Field to order results by.
            limit: Number of results to return.
            offset: Number of results to skip.

        Returns:
            FindTopicsContainer: Data container with a page of results of the search.
        """

        result = find_topics.sync(
            client=self._api_client,
            q=term if term is not None else UNSET,
            order_by=(
                get_enum_from_literal(order_by, FindTopicsOrderBy)
                if order_by is not None
                else UNSET
            ),
            limit=limit if limit is not None else UNSET,
            offset=offset if offset is not None else UNSET,
            filters=UNSET,
        )
        raise_for_error(result)
        return FindTopicsContainer(
            cast(FindTopicsBaseFindResponse, result),
            FindTopicsSchema,
            fetch_method=self.find,
            fetch_method_args={
                "term": term,
                "order_by": order_by,
            },
            web_app_search_result_url=(
                _build_web_app_find_topics_url(
                    base_url=self._get_web_app_base_url(),
                    term=term,
                )
            ),
        )

    def get(self, id: str) -> GetTopicContainer:
        """Get topic by ID.

        Args:
            id: The ID of the topic to retrieve.

        Returns:
            GetTopicContainer: Data container with the topic information.
        """

        result = get_topic.sync(
            client=self._api_client,
            id=id,
        )
        raise_for_error(result)
        return GetTopicContainer(
            result,
            FindTopicsSchema,
            web_app_search_result_url=_build_web_app_get_topic_url(
                base_url=self._get_web_app_base_url(),
                id=id,
            ),
        )


def _build_web_app_find_topics_url(
    base_url: str,
    term: str | None = None,
) -> str:
    query_params = {
        "q": term,
    }
    query_string = "&".join(
        f"{key}={value}" for key, value in query_params.items() if value is not None
    )
    url = f"{base_url}/topics"
    return f"{url}?{query_string}" if query_string else url


def _build_web_app_get_topic_url(
    base_url: str,
    id: str,
) -> str:
    return f"{base_url}/topics/{id}"



from enum import Enum
from typing import Literal


class FindMediaSourcesOrderBy(str, Enum):
    COUNTISSUES = "countIssues"
    FIRSTISSUE = "firstIssue"
    LASTISSUE = "lastIssue"
    NAME = "name"
    VALUE_1 = "-name"
    VALUE_3 = "-firstIssue"
    VALUE_5 = "-lastIssue"
    VALUE_7 = "-countIssues"

    def __str__(self) -> str:
        return str(self.value)


FindMediaSourcesOrderByLiteral = Literal[
    "countIssues",
    "firstIssue",
    "lastIssue",
    "name",
    "-name",
    "-firstIssue",
    "-lastIssue",
    "-countIssues",
]



from typing import Any, Callable, Iterator, cast
from pandas import DataFrame, json_normalize

from impresso.api_client.api.media_sources import find_media_sources
from impresso.api_client.models.find_media_sources_base_find_response import (
    FindMediaSourcesBaseFindResponse,
)
from impresso.api_client.models.find_media_sources_order_by import (
    FindMediaSourcesOrderBy,
    FindMediaSourcesOrderByLiteral,
)
from impresso.api_client.models.find_media_sources_type import (
    FindMediaSourcesType,
    FindMediaSourcesTypeLiteral,
)
from impresso.api_client.types import UNSET
from impresso.api_models import BaseFind, MediaSource
from impresso.data_container import DataContainer, iterate_pages
from impresso.resources.base import Resource
from impresso.util.error import raise_for_error
from impresso.util.py import get_enum_from_literal


class FindMediaSourcesSchema(BaseFind):
    """Schema for the find media sources response."""

    data: list[MediaSource]


class FindMediaSourcesContainer(DataContainer):
    """Response of a search call."""

    def __init__(
        self,
        data: FindMediaSourcesBaseFindResponse,
        pydantic_model: type[FindMediaSourcesSchema],
        fetch_method: Callable[..., "FindMediaSourcesContainer"],
        fetch_method_args: dict[str, Any],
        web_app_search_result_url: str | None = None,
    ):
        super().__init__(data, pydantic_model, web_app_search_result_url)
        self._fetch_method = fetch_method
        self._fetch_method_args = fetch_method_args

    @property
    def df(self) -> DataFrame:
        """Return the data as a pandas dataframe."""
        data = self._data.to_dict()["data"]
        if len(data):
            return json_normalize(self._data.to_dict()["data"]).set_index("uid")
        return DataFrame()

    def pages(self) -> Iterator["FindMediaSourcesContainer"]:
        yield self
        yield from iterate_pages(
            self._fetch_method,
            self._fetch_method_args,
            self.offset,
            self.limit,
            self.total,
        )


class MediaSourcesResource(Resource):
    """Search media sources in the Impresso database.

    Media sources are newspapers and other publications available in Impresso.

    Examples:
        Find all media sources:
        >>> results = media_sources.find()  # doctest: +SKIP
        >>> print(results.df)  # doctest: +SKIP

        Search media sources by name:
        >>> results = media_sources.find(term="Gazette")  # doctest: +SKIP
        >>> print(results.df)  # doctest: +SKIP

        Filter media sources by type:
        >>> results = media_sources.find(type="newspaper")  # doctest: +SKIP
        >>> print(results.df)  # doctest: +SKIP

        Get media sources with detailed properties:
        >>> results = media_sources.find(with_properties=True)  # doctest: +SKIP
        >>> print(results.df)  # doctest: +SKIP
    """

    name = "media_sources"

    def find(
        self,
        term: str | None = None,
        type: FindMediaSourcesTypeLiteral | None = None,
        order_by: FindMediaSourcesOrderByLiteral | None = None,
        with_properties: bool = False,
        limit: int | None = None,
        offset: int | None = None,
    ) -> FindMediaSourcesContainer:
        """
        Search media sources in Impresso.

        Args:
            term: Search term.
            type: Type of media sources to search for.
            order_by: Field to order results by.
            with_properties: Include properties in the results.
            limit: Number of results to return.
            offset: Number of results to skip.

        Returns:
            FindMediaSourcesContainer: Data container with a page of results of the search.
        """
        result = find_media_sources.sync(
            client=self._api_client,
            term=term if term is not None else UNSET,
            order_by=(
                get_enum_from_literal(order_by, FindMediaSourcesOrderBy)
                if order_by is not None
                else UNSET
            ),
            type=(
                get_enum_from_literal(type, FindMediaSourcesType)
                if type is not None
                else UNSET
            ),
            include_properties=with_properties,
            limit=limit if limit is not None else UNSET,
            offset=offset if offset is not None else UNSET,
        )
        raise_for_error(result)
        return FindMediaSourcesContainer(
            cast(FindMediaSourcesBaseFindResponse, result),
            FindMediaSourcesSchema,
            fetch_method=self.find,
            fetch_method_args={
                "term": term,
                "type": type,
                "order_by": order_by,
                "with_properties": with_properties,
            },
            web_app_search_result_url=_build_web_app_media_sources_url(
                base_url=self._get_web_app_base_url(),
                term=term,
                order_by=order_by,
            ),
        )


def _build_web_app_media_sources_url(
    base_url: str,
    term: str | None = None,
    order_by: FindMediaSourcesOrderByLiteral | None = None,
) -> str:
    query_params = {
        "orderBy": order_by,
        "q": term,
    }
    query_string = "&".join(
        f"{key}={value}" for key, value in query_params.items() if value is not None
    )
    url = f"{base_url}/newspapers"
    return f"{url}?{query_string}" if query_string else url



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: query.proto
# Protobuf Python Version: 6.32.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    32,
    1,
    '',
    'query.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0bquery.proto\x12\x0eimpresso.query\"%\n\tDateRange\x12\x0c\n\x04\x66rom\x18\x01 \x01(\x12\x12\n\n\x02to\x18\x02 \x01(\x12\"\x8d\x02\n\x06\x46ilter\x12.\n\x07\x63ontext\x18\x01 \x01(\x0e\x32\x1d.impresso.query.FilterContext\x12*\n\x02op\x18\x02 \x01(\x0e\x32\x1e.impresso.query.FilterOperator\x12(\n\x04type\x18\x03 \x01(\x0e\x32\x1a.impresso.query.FilterType\x12\x32\n\tprecision\x18\x04 \x01(\x0e\x32\x1f.impresso.query.FilterPrecision\x12\t\n\x01q\x18\x05 \x03(\t\x12\x30\n\tdaterange\x18\x06 \x01(\x0b\x32\x19.impresso.query.DateRangeB\x02\x18\x01\x12\x0c\n\x04uids\x18\x07 \x03(\t\"d\n\x0bSearchQuery\x12\'\n\x07\x66ilters\x18\x01 \x03(\x0b\x32\x16.impresso.query.Filter\x12,\n\x08group_by\x18\x02 \x01(\x0e\x32\x1a.impresso.query.GroupValue\"\x86\x03\n\x1e\x43ollectionRecommenderParameter\x12R\n\x03key\x18\x01 \x01(\x0e\x32\x45.impresso.query.CollectionRecommenderParameter.RecommenderParameterId\x12\x14\n\x0cstring_value\x18\x02 \x01(\t\x12\x14\n\x0cnumber_value\x18\x03 \x01(\x11\x12\x12\n\nbool_value\x18\x04 \x01(\x08\"\xcf\x01\n\x16RecommenderParameterId\x12\x12\n\x0eID_UNSPECIFIED\x10\x00\x12\x11\n\rID_COUNT_TYPE\x10\x01\x12\x16\n\x12ID_MIN_OCCURRENCES\x10\x02\x12\x15\n\x11ID_NUMBER_TO_KEEP\x10\x03\x12\x1d\n\x19ID_REMOVE_FULLY_MENTIONED\x10\x04\x12\x1a\n\x16ID_NORMALIZE_MAX_SCORE\x10\x05\x12\r\n\tID_MARGIN\x10\x06\x12\x15\n\x11ID_SCALING_FACTOR\x10\x07\"\xc1\x02\n\x15\x43ollectionRecommender\x12\x43\n\x04type\x18\x01 \x01(\x0e\x32\x35.impresso.query.CollectionRecommender.RecommenderType\x12\x0e\n\x06weight\x18\x02 \x01(\x11\x12\x42\n\nparameters\x18\x03 \x03(\x0b\x32..impresso.query.CollectionRecommenderParameter\x12\x0f\n\x07\x65nabled\x18\x04 \x01(\x08\"~\n\x0fRecommenderType\x12\x14\n\x10TYPE_UNSPECIFIED\x10\x00\x12\x13\n\x0fTYPE_TIME_RANGE\x10\x01\x12\x11\n\rTYPE_ENTITIES\x10\x02\x12\x0f\n\x0bTYPE_TOPICS\x10\x03\x12\x1c\n\x18TYPE_TEXT_REUSE_CLUSTERS\x10\x04\"]\n\x1e\x43ollectionRecommendersSettings\x12;\n\x0crecommenders\x18\x01 \x03(\x0b\x32%.impresso.query.CollectionRecommender*R\n\rFilterContext\x12\x17\n\x13\x43ONTEXT_UNSPECIFIED\x10\x00\x12\x13\n\x0f\x43ONTEXT_INCLUDE\x10\x01\x12\x13\n\x0f\x43ONTEXT_EXCLUDE\x10\x02*M\n\x0e\x46ilterOperator\x12\x18\n\x14OPERATOR_UNSPECIFIED\x10\x00\x12\x10\n\x0cOPERATOR_AND\x10\x01\x12\x0f\n\x0bOPERATOR_OR\x10\x02*\xbc\x07\n\nFilterType\x12\x14\n\x10TYPE_UNSPECIFIED\x10\x00\x12\x0c\n\x08TYPE_UID\x10\x01\x12\x1a\n\x16TYPE_HAS_TEXT_CONTENTS\x10\x02\x12\x0e\n\nTYPE_TITLE\x10\x03\x12\x11\n\rTYPE_IS_FRONT\x10\x04\x12\r\n\tTYPE_PAGE\x10\x05\x12\x0e\n\nTYPE_ISSUE\x10\x06\x12\x0f\n\x0bTYPE_STRING\x10\x07\x12\x0f\n\x0bTYPE_ENTITY\x10\x08\x12\x12\n\x0eTYPE_NEWSPAPER\x10\t\x12\x12\n\x0eTYPE_DATERANGE\x10\n\x12\r\n\tTYPE_YEAR\x10\x0b\x12\x11\n\rTYPE_LANGUAGE\x10\x0c\x12\r\n\tTYPE_TYPE\x10\r\x12\x0e\n\nTYPE_REGEX\x10\x0e\x12\x10\n\x0cTYPE_MENTION\x10\x0f\x12\x0f\n\x0bTYPE_PERSON\x10\x10\x12\x11\n\rTYPE_LOCATION\x10\x11\x12\x0e\n\nTYPE_TOPIC\x10\x12\x12\x13\n\x0fTYPE_COLLECTION\x10\x13\x12\x14\n\x10TYPE_OCR_QUALITY\x10\x14\x12\x17\n\x13TYPE_CONTENT_LENGTH\x10\x15\x12\x10\n\x0cTYPE_COUNTRY\x10\x16\x12\x15\n\x11TYPE_ACCESS_RIGHT\x10\x17\x12\x10\n\x0cTYPE_PARTNER\x10\x18\x12\x0e\n\nTYPE_MONTH\x10\x19\x12 \n\x1cTYPE_TEXT_REUSE_CLUSTER_SIZE\x10\x1a\x12+\n\'TYPE_TEXT_REUSE_CLUSTER_LEXICAL_OVERLAP\x10\x1b\x12%\n!TYPE_TEXT_REUSE_CLUSTER_DAY_DELTA\x10\x1c\x12\x1b\n\x17TYPE_TEXT_REUSE_CLUSTER\x10\x1d\x12\x19\n\x15TYPE_MENTION_FUNCTION\x10\x1e\x12\x0c\n\x08TYPE_NAG\x10\x1f\x12\x14\n\x10TYPE_WIKIDATA_ID\x10 \x12\x14\n\x10TYPE_DATA_DOMAIN\x10!\x12\x12\n\x0eTYPE_COPYRIGHT\x10\"\x12\x14\n\x10TYPE_SOURCE_TYPE\x10#\x12\x16\n\x12TYPE_SOURCE_MEDIUM\x10$\x12\x15\n\x11TYPE_ORGANISATION\x10%\x12\x12\n\x0eTYPE_EMBEDDING\x10&\x12\x1d\n\x19TYPE_IMAGE_VISUAL_CONTENT\x10\'\x12\x18\n\x14TYPE_IMAGE_TECHNIQUE\x10(\x12!\n\x1dTYPE_IMAGE_COMMUNICATION_GOAL\x10)\x12\x1b\n\x17TYPE_IMAGE_CONTENT_TYPE\x10**\x81\x01\n\x0f\x46ilterPrecision\x12\x19\n\x15PRECISION_UNSPECIFIED\x10\x00\x12\x13\n\x0fPRECISION_EXACT\x10\x01\x12\x15\n\x11PRECISION_PARTIAL\x10\x02\x12\x13\n\x0fPRECISION_FUZZY\x10\x03\x12\x12\n\x0ePRECISION_SOFT\x10\x04*A\n\nGroupValue\x12\x1a\n\x16GROUPVALUE_UNSPECIFIED\x10\x00\x12\x17\n\x13GROUPVALUE_ARTICLES\x10\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'query_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_FILTER'].fields_by_name['daterange']._loaded_options = None
  _globals['_FILTER'].fields_by_name['daterange']._serialized_options = b'\030\001'
  _globals['_FILTERCONTEXT']._serialized_start=1256
  _globals['_FILTERCONTEXT']._serialized_end=1338
  _globals['_FILTEROPERATOR']._serialized_start=1340
  _globals['_FILTEROPERATOR']._serialized_end=1417
  _globals['_FILTERTYPE']._serialized_start=1420
  _globals['_FILTERTYPE']._serialized_end=2376
  _globals['_FILTERPRECISION']._serialized_start=2379
  _globals['_FILTERPRECISION']._serialized_end=2508
  _globals['_GROUPVALUE']._serialized_start=2510
  _globals['_GROUPVALUE']._serialized_end=2575
  _globals['_DATERANGE']._serialized_start=31
  _globals['_DATERANGE']._serialized_end=68
  _globals['_FILTER']._serialized_start=71
  _globals['_FILTER']._serialized_end=340
  _globals['_SEARCHQUERY']._serialized_start=342
  _globals['_SEARCHQUERY']._serialized_end=442
  _globals['_COLLECTIONRECOMMENDERPARAMETER']._serialized_start=445
  _globals['_COLLECTIONRECOMMENDERPARAMETER']._serialized_end=835
  _globals['_COLLECTIONRECOMMENDERPARAMETER_RECOMMENDERPARAMETERID']._serialized_start=628
  _globals['_COLLECTIONRECOMMENDERPARAMETER_RECOMMENDERPARAMETERID']._serialized_end=835
  _globals['_COLLECTIONRECOMMENDER']._serialized_start=838
  _globals['_COLLECTIONRECOMMENDER']._serialized_end=1159
  _globals['_COLLECTIONRECOMMENDER_RECOMMENDERTYPE']._serialized_start=1033
  _globals['_COLLECTIONRECOMMENDER_RECOMMENDERTYPE']._serialized_end=1159
  _globals['_COLLECTIONRECOMMENDERSSETTINGS']._serialized_start=1161
  _globals['_COLLECTIONRECOMMENDERSSETTINGS']._serialized_end=1254
# @@protoc_insertion_point(module_scope)



from typing import TYPE_CHECKING, Any, Dict, Type, TypeVar

from attrs import define as _attrs_define

if TYPE_CHECKING:
    from ..models.authentication_response_authentication import AuthenticationResponseAuthentication
    from ..models.authentication_response_user import AuthenticationResponseUser


T = TypeVar("T", bound="AuthenticationResponse")


@_attrs_define
class AuthenticationResponse:
    """Authentication Response

    Attributes:
        access_token (str):
        authentication (AuthenticationResponseAuthentication):
        user (AuthenticationResponseUser): User details
    """

    access_token: str
    authentication: "AuthenticationResponseAuthentication"
    user: "AuthenticationResponseUser"

    def to_dict(self) -> Dict[str, Any]:
        access_token = self.access_token

        authentication = self.authentication.to_dict()

        user = self.user.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "accessToken": access_token,
                "authentication": authentication,
                "user": user,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.authentication_response_authentication import AuthenticationResponseAuthentication
        from ..models.authentication_response_user import AuthenticationResponseUser

        d = src_dict.copy()
        access_token = d.pop("accessToken")

        authentication = AuthenticationResponseAuthentication.from_dict(d.pop("authentication"))

        user = AuthenticationResponseUser.from_dict(d.pop("user"))

        authentication_response = cls(
            access_token=access_token,
            authentication=authentication,
            user=user,
        )

        return authentication_response



from enum import Enum
from typing import Literal


class SearchOrderBy(str, Enum):
    DATE = "date"
    ID = "id"
    RELEVANCE = "relevance"
    VALUE_0 = "-date"
    VALUE_2 = "-relevance"
    VALUE_5 = "-id"

    def __str__(self) -> str:
        return str(self.value)


SearchOrderByLiteral = Literal[
    "date",
    "id",
    "relevance",
    "-date",
    "-relevance",
    "-id",
]



from enum import Enum
from typing import Literal


class WikidataPersonType(str, Enum):
    HUMAN = "human"

    def __str__(self) -> str:
        return str(self.value)


WikidataPersonTypeLiteral = Literal["human",]



from typing import TypeVar

from impresso.api_client.models.error import Error as ApiError
from impresso.api_models import Error

IT = TypeVar("IT")


class ImpressoError(Exception):
    """Impresso API Exception"""

    def __init__(self, error: Error):
        self.error = error
        super().__init__(str(self.error))

    def __str__(self):
        return str(self.error)


def raise_for_error(result: ApiError | IT) -> IT:
    """Raise an Impresso API Exception if the result is an error."""

    if isinstance(result, ApiError):
        error = Error.model_validate(result.to_dict())
        raise ImpressoError(error)
    else:
        return result



# generated by datamodel-codegen:
#   filename:  http://localhost:3030/swagger.json

from __future__ import annotations

from datetime import date
from typing import Any, Mapping, Optional, Sequence, Union

from pydantic import AnyUrl, AwareDatetime, BaseModel, ConfigDict, Field, RootModel
from typing_extensions import Annotated, Literal


class Q(RootModel[str]):
    root: Annotated[str, Field(max_length=6000, min_length=2)]


class QItem(RootModel[str]):
    root: Annotated[str, Field(max_length=6000, min_length=2)]


class Filter(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    context: Optional[Literal['include', 'exclude']] = 'include'
    op: Optional[Literal['AND', 'OR']] = 'OR'
    type: Annotated[
        str,
        Field(
            description="Possible values are in 'search.validators:eachFilterValidator.type.choices'"
        ),
    ]
    precision: Optional[Literal['fuzzy', 'soft', 'exact', 'partial']] = 'exact'
    q: Optional[Union[Q, Sequence[QItem]]] = None
    daterange: Annotated[
        Optional[str],
        Field(
            None,
            pattern='\\d{4}-\\d{2}-\\d{2}T\\d{2}:\\d{2}:\\d{2}Z TO \\d{4}-\\d{2}-\\d{2}T\\d{2}:\\d{2}:\\d{2}Z',
        ),
    ]
    uids: Optional[str] = None
    uid: Optional[str] = None


class FacetWithLabel(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    id: Annotated[str, Field(description='Unique identifier of the facet')]
    label: Annotated[str, Field(description='Label of the facet')]


class Error(BaseModel):
    type: Annotated[
        AnyUrl,
        Field(
            description='A URI reference [RFC3986] that identifies the problem type.'
        ),
    ]
    title: Annotated[
        str, Field(description='A short, human-readable summary of the problem type.')
    ]
    status: Annotated[
        int, Field(description='The HTTP status code ([RFC7231], Section 6)')
    ]
    detail: Annotated[
        Optional[str],
        Field(
            None,
            description='A human-readable explanation specific to this occurrence of the problem.',
        ),
    ]


class ImpressoNerRequest(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    text: Annotated[
        str,
        Field(
            description='Text to be processed for named entity recognition',
            max_length=3999,
            min_length=1,
        ),
    ]
    method: Annotated[
        Optional[Literal['ner', 'ner-nel', 'nel']],
        Field(
            'ner',
            description='NER method to be used: `ner` (default), `ner-nel` (named entity recognition with named entity linking) and `nel` (linking only - enclose entities in [START] [END] tags).',
        ),
    ]


class ImpressoImageEmbeddingRequest(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    searchTarget: Annotated[
        Literal['image', 'multimodal'],
        Field(description='Which embedding space the embedding is going to be used in'),
    ]
    bytes: Annotated[
        str,
        Field(
            description='Base64-encoded image bytes. JPG and PNG formats are supported.'
        ),
    ]


class ImpressoEmbeddingResponse(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    embedding: Annotated[
        str,
        Field(
            description='Embedding vector, base64-encoded with the model prefix. E.g. <model>:<base64-encoded vector>'
        ),
    ]


class AddCollectableItemsFromFilters(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    filters: Annotated[
        Sequence[Filter],
        Field(
            description='Filters to apply when selecting items to add to the collection'
        ),
    ]
    namespace: Annotated[
        Literal['search', 'tr_passages'],
        Field(
            description='Namespace to use when selecting items to add to the collection'
        ),
    ]


class CollectableItemsUpdatedResponse(BaseModel):
    totalAdded: Annotated[
        int, Field(description='Total number of items added to the collection')
    ]
    totalRemoved: Annotated[
        int, Field(description='Total number of items removed from the collection')
    ]


class NewCollectionRequest(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    name: Annotated[str, Field(max_length=50, min_length=2)]
    description: Annotated[Optional[str], Field(None, max_length=500)]
    accessLevel: Annotated[
        Optional[Literal['public', 'private']],
        Field(None, description='Access level of the collection.'),
    ]


class Authentication(BaseModel):
    strategy: Optional[str] = None
    payload: Optional[Mapping[str, Any]] = None


class User(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    id: int
    username: str
    firstname: str
    lastname: str
    isStaff: bool
    isActive: bool
    isSuperuser: bool
    uid: str


class AuthenticationCreateResponse(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    accessToken: str
    authentication: Authentication
    user: Annotated[User, Field(description='User details', title='User')]


class AuthenticationCreateRequest(BaseModel):
    model_config = ConfigDict(
        extra='allow',
    )
    strategy: Literal['local', 'jwt-app']
    email: Optional[str] = None
    password: Optional[str] = None
    accessToken: Optional[str] = None


class Offset(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    start: Annotated[int, Field(description='Start offset of the entity in the text')]
    end: Annotated[int, Field(description='End offset of the entity in the text')]


class Confidence(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    ner: Annotated[
        Optional[float],
        Field(None, description='Confidence score for the named entity recognition'),
    ]
    nel: Annotated[
        Optional[float],
        Field(None, description='Confidence score for the named entity linking'),
    ]


class Wikidata(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    id: Annotated[str, Field(description='Wikidata ID of the entity')]
    wikipediaPageName: Annotated[
        Optional[str], Field(None, description='Wikipedia page name of the entity')
    ]
    wikipediaPageUrl: Annotated[
        Optional[str], Field(None, description='Wikipedia page URL of the entity')
    ]


class ImpressoNerEntity(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    id: Annotated[str, Field(description='ID of the entity')]
    type: Annotated[
        Literal[
            'comp.demonym',
            'comp.function',
            'comp.name',
            'comp.qualifier',
            'comp.title',
            'loc',
            'loc.add.elec',
            'loc.add.phys',
            'loc.adm.nat',
            'loc.adm.reg',
            'loc.adm.sup',
            'loc.adm.town',
            'loc.fac',
            'loc.oro',
            'loc.phys.astro',
            'loc.phys.geo',
            'loc.phys.hydro',
            'loc.unk',
            'org',
            'org.adm',
            'org.ent',
            'org.ent.pressagency',
            'org.ent.pressagency.AFP',
            'org.ent.pressagency.ANSA',
            'org.ent.pressagency.AP',
            'org.ent.pressagency.APA',
            'org.ent.pressagency.ATS-SDA',
            'org.ent.pressagency.Belga',
            'org.ent.pressagency.CTK',
            'org.ent.pressagency.DDP-DAPD',
            'org.ent.pressagency.DNB',
            'org.ent.pressagency.DPA',
            'org.ent.pressagency.Domei',
            'org.ent.pressagency.Europapress',
            'org.ent.pressagency.Extel',
            'org.ent.pressagency.Havas',
            'org.ent.pressagency.Kipa',
            'org.ent.pressagency.Reuters',
            'org.ent.pressagency.SPK-SMP',
            'org.ent.pressagency.Stefani',
            'org.ent.pressagency.TASS',
            'org.ent.pressagency.UP-UPI',
            'org.ent.pressagency.Wolff',
            'org.ent.pressagency.Xinhua',
            'org.ent.pressagency.ag',
            'org.ent.pressagency.unk',
            'pers',
            'pers.coll',
            'pers.ind',
            'pers.ind.articleauthor',
            'prod',
            'prod.doctr',
            'prod.media',
            'time',
            'time.date.abs',
            'time.hour.abs',
            'unk',
        ],
        Field(description='Type of the entity'),
    ]
    surfaceForm: Annotated[
        Optional[str], Field(None, description='Surface form of the entity')
    ]
    offset: Optional[Offset] = None
    isTypeNested: Annotated[
        Optional[bool], Field(None, description='Whether the entity type is nested')
    ]
    confidence: Confidence
    wikidata: Optional[Wikidata] = None
    function: Annotated[
        Optional[str], Field(None, description='Function of the entity')
    ]
    name: Annotated[Optional[str], Field(None, description='Name of the entity')]


class Params(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    id: Annotated[Optional[str], Field(None, description='The collection id')]
    status: Annotated[
        Optional[Literal['DEL']], Field(None, description='The status of the operation')
    ]


class Task(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    task_id: Annotated[Optional[str], Field(None, description='The ID of the task')]
    creationDate: Annotated[
        Optional[str], Field(None, description='When task was created')
    ]


class CollectionsRemoveResponse(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    params: Params
    task: Annotated[Task, Field(description='Deletion task details')]


class UpdateCollectableItemsRequest(BaseModel):
    add: Annotated[
        Optional[Sequence[str]],
        Field(None, description='IDs of the items to add to the collection'),
    ]
    remove: Annotated[
        Optional[Sequence[str]],
        Field(None, description='IDs of the items to remove from the collection'),
    ]


class ImpressoTextEmbeddingRequest(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    searchTarget: Annotated[
        Literal['multimodal', 'text'],
        Field(description='Which embedding space the embedding is going to be used in'),
    ]
    text: Annotated[str, Field(description='Text to be embedded', max_length=8000)]


class NamedEntity(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    uid: Annotated[str, Field(description='Unique identifier of the entity')]
    count: Annotated[
        Optional[float],
        Field(None, description='How many times it is mentioned in the text'),
    ]


class Newspaper(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    uid: Annotated[str, Field(description='The unique identifier of the newspaper.')]
    title: Annotated[
        Optional[str], Field(None, description='The title of the newspaper.')
    ]
    startYear: Annotated[
        Optional[float],
        Field(
            None,
            description='The year of the first available article in the newspaper.',
            ge=0.0,
        ),
    ]
    endYear: Annotated[
        Optional[float],
        Field(
            None,
            description='The year of the last available article in the newspaper.',
            ge=0.0,
        ),
    ]
    languageCodes: Annotated[
        Optional[Sequence[str]],
        Field(None, description='ISO 639-1 codes of languages used in the newspaper.'),
    ]
    totalArticles: Annotated[
        Optional[float],
        Field(None, description='Total number of articles in the newspaper.', ge=0.0),
    ]
    totalIssues: Annotated[
        Optional[float],
        Field(None, description='Total number of issues in the newspaper.', ge=0.0),
    ]
    totalPages: Annotated[
        Optional[float],
        Field(None, description='Total number of pages in the newspaper.', ge=0.0),
    ]


class Freeform(BaseModel):
    pass
    model_config = ConfigDict(
        extra='allow',
    )


class NamedEntities(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    locations: Annotated[
        Optional[Sequence[NamedEntity]],
        Field(
            None, description='Linked location entities mentioned in the content item.'
        ),
    ]
    persons: Annotated[
        Optional[Sequence[NamedEntity]],
        Field(
            None, description='Linked person entities mentioned in the content item.'
        ),
    ]
    organisations: Annotated[
        Optional[Sequence[NamedEntity]],
        Field(
            None,
            description='Linked organisation entities mentioned in the content item.',
        ),
    ]
    newsAgencies: Annotated[
        Optional[Sequence[NamedEntity]],
        Field(
            None,
            description='Linked news agency entities mentioned in the content item.',
        ),
    ]


class ExperimentInfo(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    id: Annotated[str, Field(description='The unique identifier of the experiment.')]
    name: Annotated[str, Field(description='The display name of the experiment.')]
    description: Annotated[
        Optional[str],
        Field(None, description='A description of what the experiment does.'),
    ]


class Offset1(BaseModel):
    start: Annotated[
        int, Field(description='Start offset of the passage in the content item.')
    ]
    end: Annotated[
        int, Field(description='End offset of the passage in the content item.')
    ]


class TextReusePassage(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    uid: Annotated[str, Field(description='Unique ID of the text reuse passage.')]
    content: Annotated[
        Optional[str], Field(None, description='Textual content of the passage.')
    ]
    contentItemId: Annotated[
        Optional[str],
        Field(
            None,
            description='ID of the content item that the text reuse passage belongs to.',
        ),
    ]
    offset: Annotated[
        Optional[Offset1],
        Field(
            None,
            description='Start and end offsets of the passage in the content item.',
        ),
    ]


class SearchFacetBucket(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    count: Annotated[int, Field(description='Number of items in the bucket', ge=0)]
    value: Annotated[
        Union[str, float, int], Field(description='Value that represents the bucket.')
    ]
    label: Annotated[
        Optional[str], Field(None, description='Label of the value, if relevant.')
    ]


class VersionDetails(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    version: Annotated[str, Field(description='Version of the API.')]


class TimeCoverage(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    startDate: Annotated[
        date,
        Field(
            description='Publication date of the earliest content item in the cluster.'
        ),
    ]
    endDate: Annotated[
        date,
        Field(
            description='Publication date of the latest content item in the cluster.'
        ),
    ]


class TextReuseCluster(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    uid: Annotated[str, Field(description='Unique ID of the text reuse cluster.')]
    lexicalOverlap: Annotated[
        Optional[float],
        Field(
            None,
            description='Overlap in percents between the passages in the cluster.',
            ge=0.0,
            le=100.0,
        ),
    ]
    clusterSize: Annotated[
        Optional[int],
        Field(None, description='Number of passages in the cluster.', ge=0),
    ]
    textSample: Annotated[
        Optional[str],
        Field(
            None,
            description='Sample of a text from one of the passages in the cluster.',
        ),
    ]
    timeCoverage: Annotated[
        Optional[TimeCoverage], Field(None, description='Time coverage of the cluster.')
    ]


class PageNumber(RootModel[float]):
    root: Annotated[float, Field(ge=1.0)]


class Coordinates(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    latitude: Annotated[
        Optional[float], Field(None, description='The latitude of the location')
    ]
    longitude: Annotated[
        Optional[float], Field(None, description='The longitude of the location')
    ]


class WikidataLocation(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    id: Annotated[
        str,
        Field(
            description='The Q Wikidata ID of the location (https://www.wikidata.org/wiki/Wikidata:Identifiers)'
        ),
    ]
    type: Annotated[Literal['location'], Field(description='The type of the entity')]
    labels: Annotated[
        Optional[Mapping[str, Sequence[str]]],
        Field(None, description='Labels of the location in different languages'),
    ]
    descriptions: Annotated[
        Optional[Mapping[str, Sequence[str]]],
        Field(None, description='Descriptions of the location in different languages'),
    ]
    coordinates: Optional[Coordinates] = None


class TopicWord(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    w: Annotated[str, Field(description='Word surface form')]
    p: Annotated[float, Field(description='Probability of the word in topic')]
    h: Annotated[Optional[bool], Field(None, description='If word is highlighted')]


class WikidataHuman(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    id: Annotated[
        str,
        Field(
            description='The Q Wikidata ID of the person (https://www.wikidata.org/wiki/Wikidata:Identifiers)'
        ),
    ]
    type: Annotated[Literal['human'], Field(description='The type of the entity')]
    labels: Annotated[
        Optional[Mapping[str, Sequence[str]]],
        Field(None, description='Labels of the person in different languages'),
    ]
    descriptions: Annotated[
        Optional[Mapping[str, Sequence[str]]],
        Field(None, description='Descriptions of the person in different languages'),
    ]
    birthDate: Annotated[
        Optional[AwareDatetime], Field(None, description='The birth date of the person')
    ]
    deathDate: Annotated[
        Optional[AwareDatetime], Field(None, description='The death date of the person')
    ]
    birthPlace: Annotated[
        Optional[WikidataLocation],
        Field(None, description='The birth place of the person'),
    ]
    deathPlace: Annotated[
        Optional[WikidataLocation],
        Field(None, description='The death place of the person'),
    ]


class Collection(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    uid: Annotated[str, Field(description='Unique identifier of the collection.')]
    title: Annotated[Optional[str], Field(None, description='Title of the collection.')]
    description: Annotated[
        Optional[str], Field(None, description='Description of the collection.')
    ]
    accessLevel: Annotated[
        Optional[Literal['public', 'private']],
        Field(None, description='Access level of the collection.'),
    ]
    createdAt: Annotated[
        Optional[AwareDatetime],
        Field(None, description='Creation date of the collection.'),
    ]
    updatedAt: Annotated[
        Optional[AwareDatetime],
        Field(None, description='Last update date of the collection.'),
    ]
    totalItems: Annotated[
        Optional[int],
        Field(None, description='Total number of items in the collection.'),
    ]


class WordMatch(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    id: Annotated[str, Field(description='Unique identifier for the word')]
    languageCode: Annotated[str, Field(description='The language code of the word')]
    word: Annotated[str, Field(description='The word')]


class Name(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    langCode: Annotated[str, Field(description='ISO 639-1 language code.')]
    name: Annotated[
        str, Field(description='Name of the data provider in this language.')
    ]


class DataProvider(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    id: Annotated[str, Field(description='The unique identifier of the data provider.')]
    name: Annotated[str, Field(description='The default name of the data provider.')]
    names: Annotated[
        Sequence[Name],
        Field(description='Names of the data provider in different languages.'),
    ]


class ImageTypes(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    visualContent: Annotated[
        Optional[str],
        Field(None, description='Whether the content is an image or not.'),
    ]
    technique: Annotated[
        Optional[str],
        Field(None, description='Determines if the image is a photograph.'),
    ]
    communicationGoal: Annotated[
        Optional[str],
        Field(None, description='Purpose or communicative function of the image.'),
    ]
    visualContentType: Annotated[
        Optional[str], Field(None, description='Classification of the visual content.')
    ]


class MediaSourceRef(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    uid: Annotated[str, Field(description='The unique identifier of the media source')]
    name: Annotated[str, Field(description='The name of the media source')]
    type: Annotated[
        Optional[Literal['newspaper']],
        Field(None, description='The type of the media source'),
    ]


class Image(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    uid: Annotated[str, Field(description='The unique identifier of the image')]
    caption: Annotated[Optional[str], Field(None, description='Image caption')]
    issueUid: Annotated[
        str,
        Field(
            description='The unique identifier of the issue that the image belongs to.'
        ),
    ]
    contentItemUid: Annotated[
        Optional[str],
        Field(
            None,
            description='The unique identifier of the content item that the image belongs to.',
        ),
    ]
    previewUrl: Annotated[AnyUrl, Field(description='The URL of the image preview')]
    pageNumbers: Annotated[
        Optional[Sequence[int]],
        Field(
            None, description='The page numbers of the issue that the image belongs to.'
        ),
    ]
    imageTypes: Optional[ImageTypes] = None
    mediaSourceRef: Annotated[
        MediaSourceRef, Field(description='The media source of the image')
    ]
    date: Annotated[
        date,
        Field(
            description='The date of the image or the date of the issue that the image belongs to.'
        ),
    ]
    embeddings: Annotated[
        Optional[Sequence[str]],
        Field(
            None,
            description='Precomputed embeddings for the image in the format: <model_type>:<base64_embedding_vector>.',
        ),
    ]


class TopicMention(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    uid: Annotated[str, Field(description='Unique identifier of the topic.')]
    relevance: Annotated[
        Optional[float],
        Field(
            None,
            description='Relevance of the topic in the content item.',
            ge=0.0,
            le=1.0,
        ),
    ]


class Pagination(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    total: Annotated[
        int, Field(description='The total number of items matching the query')
    ]
    limit: Annotated[
        int, Field(description='The number of items returned in this response')
    ]
    offset: Annotated[
        int,
        Field(
            description='Starting index of the items subset returned in this response'
        ),
    ]


class BaseFind(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    data: Sequence
    pagination: Pagination


class Totals(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    articles: Annotated[
        Optional[int],
        Field(None, description='The number of articles in the media source.'),
    ]
    issues: Annotated[
        Optional[int],
        Field(None, description='The number of issues in the media source.'),
    ]
    pages: Annotated[
        Optional[int],
        Field(None, description='The number of pages in the media source.'),
    ]


class Property(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    id: Annotated[str, Field(description='The unique identifier of the property.')]
    label: Annotated[str, Field(description='The name of the property.')]
    value: Annotated[str, Field(description='The value of the property.')]


class MediaSource(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    uid: Annotated[str, Field(description='The unique identifier of the media source.')]
    type: Annotated[
        Literal['newspaper'], Field(description='The type of the media source.')
    ]
    name: Annotated[str, Field(description='A display name of the media source.')]
    languageCodes: Annotated[
        Sequence[str],
        Field(description='ISO 639-2 language codes this media source has content in.'),
    ]
    publishedPeriodYears: Annotated[
        Optional[Sequence[int]],
        Field(
            None,
            description='The range of years this media source has been published for. Impresso may not have data for all this period. Is not defined if there is no information.',
            max_length=2,
            min_length=2,
        ),
    ]
    availableDatesRange: Annotated[
        Optional[Sequence[AwareDatetime]],
        Field(
            None,
            description='The range of dates this media source has content items for. This represents the earliest and the latest dates of the contet items.  Is not defined if there are no content items for this source.',
            max_length=2,
            min_length=2,
        ),
    ]
    totals: Totals
    properties: Optional[Sequence[Property]] = None


class EntityMention(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    surfaceForm: Annotated[
        str, Field(description='The surface form (label) of the entity mention')
    ]
    mentionConfidence: Annotated[
        float, Field(description='Confidence score of the entity mention')
    ]
    startOffset: Annotated[
        Optional[int],
        Field(
            None, description='Start offset of the entity mention in the content item'
        ),
    ]
    endOffset: Annotated[
        Optional[int],
        Field(None, description='End offset of the entity mention in the content item'),
    ]


class ImpressoNerResponse(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    modelId: Annotated[
        str, Field(description='ID of the model used for the named entity recognition')
    ]
    text: Annotated[
        str, Field(description='Text processed for named entity recognition')
    ]
    timestamp: Annotated[
        AwareDatetime,
        Field(description='Timestamp of when named entity recognition was performed'),
    ]
    entities: Sequence[ImpressoNerEntity]


class Topic(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    uid: Annotated[str, Field(description='The unique identifier of the topic')]
    language: Annotated[str, Field(description='The language code of the topic')]
    contentItemsCount: Annotated[
        Optional[float],
        Field(None, description='Number of content items with this topic'),
    ]
    words: Annotated[
        Optional[Sequence[TopicWord]],
        Field(None, description='Top N words associated with the topic'),
    ]
    model: Annotated[
        Optional[str],
        Field(None, description='ID of the model used to generate the topic'),
    ]


class EntityDetails(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    uid: Annotated[str, Field(description='Unique identifier of the entity')]
    label: Annotated[Optional[str], Field(None, description='Entity label')]
    type: Optional[Literal['person', 'location', 'organisation', 'newsagency']] = None
    wikidataId: Annotated[
        Optional[str], Field(None, description='Wikidata identifier of the entity.')
    ]
    totalMentions: Annotated[
        Optional[int],
        Field(None, description='Total number of mentions of the entity.'),
    ]
    totalContentItems: Annotated[
        Optional[int],
        Field(
            None,
            description='Total number of content items the entity is mentioned in.',
        ),
    ]
    wikidataDetails: Optional[Union[WikidataHuman, WikidataLocation]] = None


class EntityMentions(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    locations: Annotated[
        Optional[Sequence[EntityMention]],
        Field(None, description='Locations mentioned in the content item.'),
    ]
    persons: Annotated[
        Optional[Sequence[EntityMention]],
        Field(None, description='Persons mentioned in the content item.'),
    ]
    organisations: Annotated[
        Optional[Sequence[EntityMention]],
        Field(None, description='Organisations mentioned in the content item.'),
    ]
    newsAgencies: Annotated[
        Optional[Sequence[EntityMention]],
        Field(None, description='News agencies mentioned in the content item.'),
    ]


class ContentItem(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    uid: Annotated[
        str,
        Field(description='The unique identifier of the content item.', min_length=1),
    ]
    copyrightStatus: Annotated[
        Optional[Literal['pbl', 'und', 'nkn', 'euo', 'unk', 'in_cpy']],
        Field(None, description='Copyright status.'),
    ]
    type: Annotated[
        Optional[str],
        Field(
            None,
            description='The type of the content item, as present in the OLR provided by the data provider. All content items are not characterised by the same set of types.',
        ),
    ]
    sourceMedium: Annotated[
        Optional[Literal['audio', 'print', 'typescript']],
        Field(
            None,
            description='Medium of the source (audio for audio radio broadcasts, print for newspapers, typescript for digitised radio bulletin typescripts).',
        ),
    ]
    title: Annotated[
        Optional[str], Field(None, description='The title of the content item.')
    ]
    transcript: Annotated[
        Optional[str], Field(None, description='Transcript of the content item.')
    ]
    entities: Optional[NamedEntities] = None
    mentions: Optional[EntityMentions] = None
    topics: Annotated[
        Optional[Sequence[TopicMention]],
        Field(None, description='Topics mentioned in the content item.'),
    ]
    embeddings: Annotated[
        Optional[Sequence[str]],
        Field(
            None,
            description='Precomputed embeddings for the content item in the format: <model_type>:<base64_embedding_vector>.',
        ),
    ]
    transcriptLength: Annotated[
        Optional[float],
        Field(None, description='The length of the transcript in characters.', ge=0.0),
    ]
    totalPages: Annotated[
        Optional[float],
        Field(None, description='Total number of pages the item covers.', ge=0.0),
    ]
    languageCode: Annotated[
        Optional[str],
        Field(None, description='ISO 639-1 language code of the content item.'),
    ]
    isOnFrontPage: Annotated[
        Optional[bool],
        Field(
            None,
            description='Whether the content item is on the front page of the publication.',
        ),
    ]
    publicationDate: Annotated[
        Optional[AwareDatetime],
        Field(None, description='The publication date of the content item.'),
    ]
    issueUid: Annotated[
        Optional[str], Field(None, description='Unique issue identifier')
    ]
    countryCode: Annotated[
        Optional[str],
        Field(None, description='ISO 3166-1 alpha-2 country code of the content item.'),
    ]
    providerCode: Annotated[
        Optional[str], Field(None, description='The code of the data provider.')
    ]
    mediaUid: Annotated[
        Optional[str],
        Field(
            None,
            description='Media title alias. Usually a 3 letter code of the media title (newspaper, radio station, etc.).',
        ),
    ]
    mediaType: Annotated[
        Optional[
            Literal[
                'newspaper',
                'radio_broadcast',
                'radio_magazine',
                'radio_schedule',
                'monograph',
                'encyclopedia',
            ]
        ],
        Field(None, description='The type of the media the content item belongs to.'),
    ]
    hasOLR: Annotated[
        Optional[bool],
        Field(None, description='Whether the content item has OCR/OLR data available.'),
    ]
    ocrQualityScore: Annotated[
        Optional[float],
        Field(
            None,
            description='OCR quality score of the content item (0 - 1).',
            ge=0.0,
            le=1.0,
        ),
    ]
    relevanceScore: Annotated[
        Optional[float],
        Field(
            None,
            description='Relevance score of the content item (0 - 1).',
            ge=0.0,
            le=1.0,
        ),
    ]
    pageNumbers: Annotated[
        Optional[Sequence[PageNumber]],
        Field(None, description='Page numbers the content item appears on.'),
    ]
    collectionUids: Annotated[
        Optional[Sequence[str]],
        Field(
            None,
            description='Unique identifiers of collections the content item belongs to.',
        ),
    ]



"""Contains all the data models used in inputs/outputs"""

from .add_collectable_items_from_filters import AddCollectableItemsFromFilters
from .add_collectable_items_from_filters_namespace import AddCollectableItemsFromFiltersNamespace
from .authentication_create_request import AuthenticationCreateRequest
from .authentication_create_request_strategy import AuthenticationCreateRequestStrategy
from .authentication_response import AuthenticationResponse
from .authentication_response_authentication import AuthenticationResponseAuthentication
from .authentication_response_authentication_payload import AuthenticationResponseAuthenticationPayload
from .authentication_response_user import AuthenticationResponseUser
from .base_find_response import BaseFindResponse
from .base_find_response_pagination import BaseFindResponsePagination
from .collectable_items_updated_response import CollectableItemsUpdatedResponse
from .collection import Collection
from .collection_access_level import CollectionAccessLevel
from .content_item import ContentItem
from .content_item_copyright_status import ContentItemCopyrightStatus
from .content_item_entities_mentions_information import ContentItemEntitiesMentionsInformation
from .content_item_media_type import ContentItemMediaType
from .content_item_named_entities_information import ContentItemNamedEntitiesInformation
from .content_item_source_medium import ContentItemSourceMedium
from .data_provider import DataProvider
from .data_provider_names_item import DataProviderNamesItem
from .entity_details import EntityDetails
from .entity_details_type import EntityDetailsType
from .entity_mention import EntityMention
from .error import Error
from .experiment_info import ExperimentInfo
from .facet_with_label import FacetWithLabel
from .filter_ import Filter
from .filter_context import FilterContext
from .filter_op import FilterOp
from .filter_precision import FilterPrecision
from .find_collections_base_find_response import FindCollectionsBaseFindResponse
from .find_collections_base_find_response_pagination import FindCollectionsBaseFindResponsePagination
from .find_collections_order_by import FindCollectionsOrderBy
from .find_data_providers_base_find_response import FindDataProvidersBaseFindResponse
from .find_data_providers_base_find_response_pagination import FindDataProvidersBaseFindResponsePagination
from .find_entities_base_find_response import FindEntitiesBaseFindResponse
from .find_entities_base_find_response_pagination import FindEntitiesBaseFindResponsePagination
from .find_entities_order_by import FindEntitiesOrderBy
from .find_experiments_base_find_response import FindExperimentsBaseFindResponse
from .find_experiments_base_find_response_pagination import FindExperimentsBaseFindResponsePagination
from .find_images_base_find_response import FindImagesBaseFindResponse
from .find_images_base_find_response_pagination import FindImagesBaseFindResponsePagination
from .find_images_order_by import FindImagesOrderBy
from .find_media_sources_base_find_response import FindMediaSourcesBaseFindResponse
from .find_media_sources_base_find_response_pagination import FindMediaSourcesBaseFindResponsePagination
from .find_media_sources_order_by import FindMediaSourcesOrderBy
from .find_media_sources_type import FindMediaSourcesType
from .find_text_reuse_clusters_base_find_response import FindTextReuseClustersBaseFindResponse
from .find_text_reuse_clusters_base_find_response_pagination import FindTextReuseClustersBaseFindResponsePagination
from .find_text_reuse_clusters_order_by import FindTextReuseClustersOrderBy
from .find_text_reuse_passages_base_find_response import FindTextReusePassagesBaseFindResponse
from .find_text_reuse_passages_base_find_response_pagination import FindTextReusePassagesBaseFindResponsePagination
from .find_text_reuse_passages_order_by import FindTextReusePassagesOrderBy
from .find_topics_base_find_response import FindTopicsBaseFindResponse
from .find_topics_base_find_response_pagination import FindTopicsBaseFindResponsePagination
from .find_topics_order_by import FindTopicsOrderBy
from .freeform import Freeform
from .get_images_facet_base_find_response import GetImagesFacetBaseFindResponse
from .get_images_facet_base_find_response_pagination import GetImagesFacetBaseFindResponsePagination
from .get_images_facet_id import GetImagesFacetId
from .get_images_facet_order_by import GetImagesFacetOrderBy
from .get_search_facet_base_find_response import GetSearchFacetBaseFindResponse
from .get_search_facet_base_find_response_pagination import GetSearchFacetBaseFindResponsePagination
from .get_search_facet_id import GetSearchFacetId
from .get_search_facet_order_by import GetSearchFacetOrderBy
from .get_tr_clusters_facet_base_find_response import GetTrClustersFacetBaseFindResponse
from .get_tr_clusters_facet_base_find_response_pagination import GetTrClustersFacetBaseFindResponsePagination
from .get_tr_clusters_facet_id import GetTrClustersFacetId
from .get_tr_clusters_facet_order_by import GetTrClustersFacetOrderBy
from .get_tr_passages_facet_base_find_response import GetTrPassagesFacetBaseFindResponse
from .get_tr_passages_facet_base_find_response_pagination import GetTrPassagesFacetBaseFindResponsePagination
from .get_tr_passages_facet_id import GetTrPassagesFacetId
from .get_tr_passages_facet_order_by import GetTrPassagesFacetOrderBy
from .image import Image
from .image_image_types import ImageImageTypes
from .image_media_source_ref import ImageMediaSourceRef
from .image_media_source_ref_type import ImageMediaSourceRefType
from .impresso_embedding_response import ImpressoEmbeddingResponse
from .impresso_image_embedding_request import ImpressoImageEmbeddingRequest
from .impresso_image_embedding_request_search_target import ImpressoImageEmbeddingRequestSearchTarget
from .impresso_named_entity_recognition_entity import ImpressoNamedEntityRecognitionEntity
from .impresso_named_entity_recognition_entity_confidence import ImpressoNamedEntityRecognitionEntityConfidence
from .impresso_named_entity_recognition_entity_offset import ImpressoNamedEntityRecognitionEntityOffset
from .impresso_named_entity_recognition_entity_type import ImpressoNamedEntityRecognitionEntityType
from .impresso_named_entity_recognition_entity_wikidata import ImpressoNamedEntityRecognitionEntityWikidata
from .impresso_named_entity_recognition_request import ImpressoNamedEntityRecognitionRequest
from .impresso_named_entity_recognition_request_method import ImpressoNamedEntityRecognitionRequestMethod
from .impresso_named_entity_recognition_response import ImpressoNamedEntityRecognitionResponse
from .impresso_text_embedding_request import ImpressoTextEmbeddingRequest
from .impresso_text_embedding_request_search_target import ImpressoTextEmbeddingRequestSearchTarget
from .interact_with_experiment_body import InteractWithExperimentBody
from .media_source import MediaSource
from .media_source_properties_item import MediaSourcePropertiesItem
from .media_source_totals import MediaSourceTotals
from .media_source_type import MediaSourceType
from .named_entity import NamedEntity
from .new_collection_request import NewCollectionRequest
from .new_collection_request_access_level import NewCollectionRequestAccessLevel
from .newspaper import Newspaper
from .remove_collection_response import RemoveCollectionResponse
from .remove_collection_response_params import RemoveCollectionResponseParams
from .remove_collection_response_params_status import RemoveCollectionResponseParamsStatus
from .remove_collection_response_task import RemoveCollectionResponseTask
from .search_base_find_response import SearchBaseFindResponse
from .search_base_find_response_pagination import SearchBaseFindResponsePagination
from .search_facet_bucket import SearchFacetBucket
from .search_order_by import SearchOrderBy
from .text_reuse_cluster import TextReuseCluster
from .text_reuse_cluster_time_coverage import TextReuseClusterTimeCoverage
from .text_reuse_passage import TextReusePassage
from .text_reuse_passage_offset import TextReusePassageOffset
from .topic import Topic
from .topic_mention import TopicMention
from .topic_word import TopicWord
from .update_collectable_items_request import UpdateCollectableItemsRequest
from .version_details import VersionDetails
from .wikidata_location import WikidataLocation
from .wikidata_location_coordinates import WikidataLocationCoordinates
from .wikidata_location_descriptions import WikidataLocationDescriptions
from .wikidata_location_labels import WikidataLocationLabels
from .wikidata_location_type import WikidataLocationType
from .wikidata_person import WikidataPerson
from .wikidata_person_descriptions import WikidataPersonDescriptions
from .wikidata_person_labels import WikidataPersonLabels
from .wikidata_person_type import WikidataPersonType
from .word_match import WordMatch

__all__ = (
    "AddCollectableItemsFromFilters",
    "AddCollectableItemsFromFiltersNamespace",
    "AuthenticationCreateRequest",
    "AuthenticationCreateRequestStrategy",
    "AuthenticationResponse",
    "AuthenticationResponseAuthentication",
    "AuthenticationResponseAuthenticationPayload",
    "AuthenticationResponseUser",
    "BaseFindResponse",
    "BaseFindResponsePagination",
    "CollectableItemsUpdatedResponse",
    "Collection",
    "CollectionAccessLevel",
    "ContentItem",
    "ContentItemCopyrightStatus",
    "ContentItemEntitiesMentionsInformation",
    "ContentItemMediaType",
    "ContentItemNamedEntitiesInformation",
    "ContentItemSourceMedium",
    "DataProvider",
    "DataProviderNamesItem",
    "EntityDetails",
    "EntityDetailsType",
    "EntityMention",
    "Error",
    "ExperimentInfo",
    "FacetWithLabel",
    "Filter",
    "FilterContext",
    "FilterOp",
    "FilterPrecision",
    "FindCollectionsBaseFindResponse",
    "FindCollectionsBaseFindResponsePagination",
    "FindCollectionsOrderBy",
    "FindDataProvidersBaseFindResponse",
    "FindDataProvidersBaseFindResponsePagination",
    "FindEntitiesBaseFindResponse",
    "FindEntitiesBaseFindResponsePagination",
    "FindEntitiesOrderBy",
    "FindExperimentsBaseFindResponse",
    "FindExperimentsBaseFindResponsePagination",
    "FindImagesBaseFindResponse",
    "FindImagesBaseFindResponsePagination",
    "FindImagesOrderBy",
    "FindMediaSourcesBaseFindResponse",
    "FindMediaSourcesBaseFindResponsePagination",
    "FindMediaSourcesOrderBy",
    "FindMediaSourcesType",
    "FindTextReuseClustersBaseFindResponse",
    "FindTextReuseClustersBaseFindResponsePagination",
    "FindTextReuseClustersOrderBy",
    "FindTextReusePassagesBaseFindResponse",
    "FindTextReusePassagesBaseFindResponsePagination",
    "FindTextReusePassagesOrderBy",
    "FindTopicsBaseFindResponse",
    "FindTopicsBaseFindResponsePagination",
    "FindTopicsOrderBy",
    "Freeform",
    "GetImagesFacetBaseFindResponse",
    "GetImagesFacetBaseFindResponsePagination",
    "GetImagesFacetId",
    "GetImagesFacetOrderBy",
    "GetSearchFacetBaseFindResponse",
    "GetSearchFacetBaseFindResponsePagination",
    "GetSearchFacetId",
    "GetSearchFacetOrderBy",
    "GetTrClustersFacetBaseFindResponse",
    "GetTrClustersFacetBaseFindResponsePagination",
    "GetTrClustersFacetId",
    "GetTrClustersFacetOrderBy",
    "GetTrPassagesFacetBaseFindResponse",
    "GetTrPassagesFacetBaseFindResponsePagination",
    "GetTrPassagesFacetId",
    "GetTrPassagesFacetOrderBy",
    "Image",
    "ImageImageTypes",
    "ImageMediaSourceRef",
    "ImageMediaSourceRefType",
    "ImpressoEmbeddingResponse",
    "ImpressoImageEmbeddingRequest",
    "ImpressoImageEmbeddingRequestSearchTarget",
    "ImpressoNamedEntityRecognitionEntity",
    "ImpressoNamedEntityRecognitionEntityConfidence",
    "ImpressoNamedEntityRecognitionEntityOffset",
    "ImpressoNamedEntityRecognitionEntityType",
    "ImpressoNamedEntityRecognitionEntityWikidata",
    "ImpressoNamedEntityRecognitionRequest",
    "ImpressoNamedEntityRecognitionRequestMethod",
    "ImpressoNamedEntityRecognitionResponse",
    "ImpressoTextEmbeddingRequest",
    "ImpressoTextEmbeddingRequestSearchTarget",
    "InteractWithExperimentBody",
    "MediaSource",
    "MediaSourcePropertiesItem",
    "MediaSourceTotals",
    "MediaSourceType",
    "NamedEntity",
    "NewCollectionRequest",
    "NewCollectionRequestAccessLevel",
    "Newspaper",
    "RemoveCollectionResponse",
    "RemoveCollectionResponseParams",
    "RemoveCollectionResponseParamsStatus",
    "RemoveCollectionResponseTask",
    "SearchBaseFindResponse",
    "SearchBaseFindResponsePagination",
    "SearchFacetBucket",
    "SearchOrderBy",
    "TextReuseCluster",
    "TextReuseClusterTimeCoverage",
    "TextReusePassage",
    "TextReusePassageOffset",
    "Topic",
    "TopicMention",
    "TopicWord",
    "UpdateCollectableItemsRequest",
    "VersionDetails",
    "WikidataLocation",
    "WikidataLocationCoordinates",
    "WikidataLocationDescriptions",
    "WikidataLocationLabels",
    "WikidataLocationType",
    "WikidataPerson",
    "WikidataPersonDescriptions",
    "WikidataPersonLabels",
    "WikidataPersonType",
    "WordMatch",
)



from typing import Any, Callable, Generic, TypeVar, Iterator
from pydantic import BaseModel
from pandas import DataFrame

IT = TypeVar("IT")
T = TypeVar("T", bound=BaseModel)


class DataContainer(Generic[IT, T]):
    """
    Generic container for responses from the Impresso API
    returned by resource methods (`get`, `find`).

    Generally represents a single page of the result. The results can be
    paginated through by adjusting the `offset` and `limit` parameters
    in the corresponding resource method call (e.g., `client.newspapers.find`).
    The `total`, `limit`, `offset`, and `size` properties provide information
    about the current page and the overall result set.
    """

    def __init__(
        self,
        data: IT,
        pydantic_model: type[T],
        web_app_search_result_url: str | None = None,
    ):
        if data is None or getattr(data, "to_dict") is None:
            raise ValueError(f"Unexpected data object: {data}")
        self._data = data
        self._pydantic_model = pydantic_model
        self._web_app_search_result_url = web_app_search_result_url

    def _repr_html_(self):
        df_repr = self.df.head(3)._repr_html_()
        response_type = self.__class__.__name__.replace("DataContainer", "").replace(
            "Container", ""
        )
        preview_img = self._get_preview_image_()

        grid_template_style = (
            "grid-template-columns: minmax(200px, 1fr) auto;"
            if preview_img is not None
            else ""
        )

        items = [
            f'<div style="display: grid; {grid_template_style}">',
            "<div>",
            f"<h2>{response_type} result</h2>",
            f"<div>Contains <b>{self.size}</b> items "
            + (
                f"(<b>{self.offset}</b> - <b>{self.offset + self.size}</b>) "
                if self.size > 0 and self.size < self.total
                else ""
            )
            + f"of <b>{self.total}</b> total items.</div>",
            "<br/>",
            (
                f'See this result in the <a href="{self.url}">Impresso App</a>.'
                if self.url
                else None
            ),
            "</div>",
            (
                (
                    f'<div style="align-content: center;"><img src="data:image/png;base64,{preview_img}" '
                    + 'style="max-width: 800px; width: 100%;"></div>'
                )
                if preview_img
                else None
            ),
            "</div>",
            "<h3>Data preview:</h3>",
            df_repr,
        ]

        return "\n".join([item for item in items if item])

    def _get_preview_image_(self) -> str | None:
        return None

    @property
    def raw(self) -> dict[str, Any]:
        """The response data as a python dictionary."""
        return getattr(self._data, "to_dict")()

    @property
    def pydantic(self) -> T:
        """The response data as a pydantic model."""
        return self._pydantic_model.model_validate(self.raw)

    @property
    def df(self) -> DataFrame:
        """
        The response data for the current page as a pandas dataframe.

        Note that this DataFrame only contains the items from the current
        page of results, not the entire result set across all pages.
        """
        return DataFrame.from_dict(self._data)  # type: ignore

    @property
    def total(self) -> int:
        """Total number of results available across all pages."""
        return self.raw.get("pagination", {}).get("total", 0)

    @property
    def limit(self) -> int:
        """Maximum number of items requested for the current page."""
        return self.raw.get("pagination", {}).get("limit", 0)

    @property
    def offset(self) -> int:
        """The starting index (0-based) of the items on the current page."""
        return self.raw.get("pagination", {}).get("offset", 0)

    @property
    def size(self) -> int:
        """Number of items actually present on the current page."""
        return len(self.raw.get("data", []))

    @property
    def url(self) -> str | None:
        """
        URL of an Impresso web application page representing the result set.
        """
        return self._web_app_search_result_url

    def pages(self) -> Iterator["DataContainer[IT, T]"]:
        """
        Yields the current page and all subsequent pages of results.

        This method first yields the current DataContainer instance (self),
        then attempts to fetch and yield subsequent pages by making new API
        calls with adjusted offsets.

        Returns:
            Iterator["DataContainer[IT, T]"]: An iterator that yields
            DataContainer instances, starting with the current one,
            followed by subsequent pages.

        Example:

            # Get the first page with 10 items per page
            first_page = client.newspapers.find(limit=10)

            # Iterate through all pages
            for page in first_page.pages():
                # Process items from the current page
                print(f"Page {page.offset // page.limit + 1}:")
                print(page.df)
                # The loop will continue with the next page, if any
        """
        # Implementation Note:
        # To fully implement this method, it should first `yield self`.
        # Then, it would need access to the original function/client method
        # that fetched the current page, along with its parameters, to make
        # new calls for subsequent pages in a loop, yielding each new
        # DataContainer. This might involve modifying the __init__
        # method to store this information. The loop would continue as long
        # as `self.offset + self.size < self.total`.

        # Default implementation is suitable for `Get*` containers that
        # contain 0 or 1 items and no further pages.
        yield self  # Return the current page
        raise StopIteration


DC = TypeVar("DC", bound=DataContainer)


def iterate_pages(
    fetch_method: Callable[..., DC],
    fetch_method_args: dict[str, Any],
    initial_offset: int,
    limit: int,
    total: int,
) -> Iterator[DC]:
    """
    Iterate over the pages of results from the media sources API, starting from the
    next page.

    Args:
        fetch_method (Callable[..., DC]): The method to call to fetch each page of results.
        fetch_method_args (dict[str, Any]): The arguments to pass to the fetch method.
        initial_offset (int): The initial offset for the first page.
        limit (int): The maximum number of items to fetch per page.

    Returns:
        Iterator[DC]: An iterator over the pages of results.
    """
    offset = initial_offset + limit
    while offset < total:
        page = fetch_method(
            **fetch_method_args,
            **{"offset": offset, "limit": limit},
        )
        yield page
        offset += limit



from typing import Any, Callable, cast

from pandas import DataFrame, json_normalize

from impresso.api_client.api.experiments import (
    interact_with_experiment,
    find_experiments,
)
from impresso.api_client.models.find_experiments_base_find_response import (
    FindExperimentsBaseFindResponse,
)
from impresso.api_client.models.freeform import Freeform
from impresso.api_client.models.interact_with_experiment_body import (
    InteractWithExperimentBody,
)
from impresso.data_container import DataContainer
from impresso.resources.base import Resource
from impresso.util.error import raise_for_error

from impresso.api_models import BaseFind, ExperimentInfo


class FindExperimentsSchema(BaseFind):
    """Schema for the find experiments response."""

    data: list[ExperimentInfo]


class FindExperimentsContainer(DataContainer):
    """Response of a find call."""

    def __init__(
        self,
        data: FindExperimentsBaseFindResponse,
        pydantic_model: type[FindExperimentsSchema],
        fetch_method: Callable[..., "FindExperimentsContainer"],
        fetch_method_args: dict[str, Any],
        web_app_search_result_url: str | None = None,
    ):
        super().__init__(data, pydantic_model, web_app_search_result_url)
        self._fetch_method = fetch_method
        self._fetch_method_args = fetch_method_args

    @property
    def df(self) -> DataFrame:
        """Return the data as a pandas dataframe."""
        data = self._data.to_dict()["data"]
        if len(data):
            return json_normalize(self._data.to_dict()["data"]).set_index("id")
        return DataFrame()


class ExperimentsResource(Resource):
    """Experiment with Impresso."""

    name = "experiments"

    def find(self) -> FindExperimentsContainer:
        """Find all available experiments.

        Returns:
            FindExperimentsContainer: Data container with list of available experiments.
        """
        result = find_experiments.sync(
            client=self._api_client,
        )
        raise_for_error(result)

        return FindExperimentsContainer(
            cast(FindExperimentsBaseFindResponse, result),
            FindExperimentsSchema,
            fetch_method=self.find,
            fetch_method_args={},
        )

    def execute(self, experiment_id: str, body: dict) -> dict:
        """Execute an experiment with the given ID.

        Args:
            experiment_id: ID of the experiment to execute.
            body: Body of the experiment.

        Returns:
            dict: Result of the experiment.
        """
        result = interact_with_experiment.sync(
            client=self._api_client,
            id=experiment_id,
            body=InteractWithExperimentBody.from_dict(body),
        )
        raise_for_error(result)
        return cast(Freeform, result).to_dict()



from typing import TYPE_CHECKING, Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..models.impresso_named_entity_recognition_entity_type import ImpressoNamedEntityRecognitionEntityType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.impresso_named_entity_recognition_entity_confidence import (
        ImpressoNamedEntityRecognitionEntityConfidence,
    )
    from ..models.impresso_named_entity_recognition_entity_offset import ImpressoNamedEntityRecognitionEntityOffset
    from ..models.impresso_named_entity_recognition_entity_wikidata import ImpressoNamedEntityRecognitionEntityWikidata


T = TypeVar("T", bound="ImpressoNamedEntityRecognitionEntity")


@_attrs_define
class ImpressoNamedEntityRecognitionEntity:
    """Impresso NER entity

    Attributes:
        id (str): ID of the entity
        type (ImpressoNamedEntityRecognitionEntityType): Type of the entity
        confidence (ImpressoNamedEntityRecognitionEntityConfidence):
        surface_form (Union[Unset, str]): Surface form of the entity
        offset (Union[Unset, ImpressoNamedEntityRecognitionEntityOffset]):
        is_type_nested (Union[Unset, bool]): Whether the entity type is nested
        wikidata (Union[Unset, ImpressoNamedEntityRecognitionEntityWikidata]):
        function (Union[Unset, str]): Function of the entity
        name (Union[Unset, str]): Name of the entity
    """

    id: str
    type: ImpressoNamedEntityRecognitionEntityType
    confidence: "ImpressoNamedEntityRecognitionEntityConfidence"
    surface_form: Union[Unset, str] = UNSET
    offset: Union[Unset, "ImpressoNamedEntityRecognitionEntityOffset"] = UNSET
    is_type_nested: Union[Unset, bool] = UNSET
    wikidata: Union[Unset, "ImpressoNamedEntityRecognitionEntityWikidata"] = UNSET
    function: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        type = self.type.value

        confidence = self.confidence.to_dict()

        surface_form = self.surface_form

        offset: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.offset, Unset):
            offset = self.offset.to_dict()

        is_type_nested = self.is_type_nested

        wikidata: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.wikidata, Unset):
            wikidata = self.wikidata.to_dict()

        function = self.function

        name = self.name

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "id": id,
                "type": type,
                "confidence": confidence,
            }
        )
        if surface_form is not UNSET:
            field_dict["surfaceForm"] = surface_form
        if offset is not UNSET:
            field_dict["offset"] = offset
        if is_type_nested is not UNSET:
            field_dict["isTypeNested"] = is_type_nested
        if wikidata is not UNSET:
            field_dict["wikidata"] = wikidata
        if function is not UNSET:
            field_dict["function"] = function
        if name is not UNSET:
            field_dict["name"] = name

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.impresso_named_entity_recognition_entity_confidence import (
            ImpressoNamedEntityRecognitionEntityConfidence,
        )
        from ..models.impresso_named_entity_recognition_entity_offset import ImpressoNamedEntityRecognitionEntityOffset
        from ..models.impresso_named_entity_recognition_entity_wikidata import (
            ImpressoNamedEntityRecognitionEntityWikidata,
        )

        d = src_dict.copy()
        id = d.pop("id")

        type = ImpressoNamedEntityRecognitionEntityType(d.pop("type"))

        confidence = ImpressoNamedEntityRecognitionEntityConfidence.from_dict(d.pop("confidence"))

        surface_form = d.pop("surfaceForm", UNSET)

        _offset = d.pop("offset", UNSET)
        offset: Union[Unset, ImpressoNamedEntityRecognitionEntityOffset]
        if isinstance(_offset, Unset):
            offset = UNSET
        else:
            offset = ImpressoNamedEntityRecognitionEntityOffset.from_dict(_offset)

        is_type_nested = d.pop("isTypeNested", UNSET)

        _wikidata = d.pop("wikidata", UNSET)
        wikidata: Union[Unset, ImpressoNamedEntityRecognitionEntityWikidata]
        if isinstance(_wikidata, Unset):
            wikidata = UNSET
        else:
            wikidata = ImpressoNamedEntityRecognitionEntityWikidata.from_dict(_wikidata)

        function = d.pop("function", UNSET)

        name = d.pop("name", UNSET)

        impresso_named_entity_recognition_entity = cls(
            id=id,
            type=type,
            confidence=confidence,
            surface_form=surface_form,
            offset=offset,
            is_type_nested=is_type_nested,
            wikidata=wikidata,
            function=function,
            name=name,
        )

        return impresso_named_entity_recognition_entity



from typing import Any, Dict, Type, TypeVar

from attrs import define as _attrs_define

from ..models.impresso_image_embedding_request_search_target import ImpressoImageEmbeddingRequestSearchTarget

T = TypeVar("T", bound="ImpressoImageEmbeddingRequest")


@_attrs_define
class ImpressoImageEmbeddingRequest:
    """Body of a request to the Impresso Image Embedding endpoint

    Attributes:
        search_target (ImpressoImageEmbeddingRequestSearchTarget): Which embedding space the embedding is going to be
            used in
        bytes_ (str): Base64-encoded image bytes. JPG and PNG formats are supported.
    """

    search_target: ImpressoImageEmbeddingRequestSearchTarget
    bytes_: str

    def to_dict(self) -> Dict[str, Any]:
        search_target = self.search_target.value

        bytes_ = self.bytes_

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "searchTarget": search_target,
                "bytes": bytes_,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        search_target = ImpressoImageEmbeddingRequestSearchTarget(d.pop("searchTarget"))

        bytes_ = d.pop("bytes")

        impresso_image_embedding_request = cls(
            search_target=search_target,
            bytes_=bytes_,
        )

        return impresso_image_embedding_request



from enum import Enum
from typing import Literal


class ImpressoImageEmbeddingRequestSearchTarget(str, Enum):
    IMAGE = "image"
    MULTIMODAL = "multimodal"

    def __str__(self) -> str:
        return str(self.value)


ImpressoImageEmbeddingRequestSearchTargetLiteral = Literal[
    "image",
    "multimodal",
]



from enum import Enum
from typing import Literal


class GetImagesFacetId(str, Enum):
    COLLECTION = "collection"
    IMAGECOMMUNICATIONGOAL = "imageCommunicationGoal"
    IMAGECONTENTTYPE = "imageContentType"
    IMAGETECHNIQUE = "imageTechnique"
    IMAGEVISUALCONTENT = "imageVisualContent"
    NEWSPAPER = "newspaper"
    YEAR = "year"

    def __str__(self) -> str:
        return str(self.value)


GetImagesFacetIdLiteral = Literal[
    "collection",
    "imageCommunicationGoal",
    "imageContentType",
    "imageTechnique",
    "imageVisualContent",
    "newspaper",
    "year",
]



from typing import Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..types import UNSET, Unset

T = TypeVar("T", bound="ExperimentInfo")


@_attrs_define
class ExperimentInfo:
    """Information about an available experiment including its identifier, name, and description.

    Attributes:
        id (str): The unique identifier of the experiment.
        name (str): The display name of the experiment.
        description (Union[Unset, str]): A description of what the experiment does.
    """

    id: str
    name: str
    description: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        name = self.name

        description = self.description

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "id": id,
                "name": name,
            }
        )
        if description is not UNSET:
            field_dict["description"] = description

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        name = d.pop("name")

        description = d.pop("description", UNSET)

        experiment_info = cls(
            id=id,
            name=name,
            description=description,
        )

        return experiment_info



from enum import Enum
from typing import Literal


class ContentItemSourceMedium(str, Enum):
    AUDIO = "audio"
    PRINT = "print"
    TYPESCRIPT = "typescript"

    def __str__(self) -> str:
        return str(self.value)


ContentItemSourceMediumLiteral = Literal[
    "audio",
    "print",
    "typescript",
]



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from dateutil.parser import isoparse

from ..models.media_source_type import MediaSourceType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.media_source_properties_item import MediaSourcePropertiesItem
    from ..models.media_source_totals import MediaSourceTotals


T = TypeVar("T", bound="MediaSource")


@_attrs_define
class MediaSource:
    """A media source is what a content item belongs to. This can be a newspaper, a TV or a radio station, etc.

    Attributes:
        uid (str): The unique identifier of the media source.
        type (MediaSourceType): The type of the media source.
        name (str): A display name of the media source.
        language_codes (List[str]): ISO 639-2 language codes this media source has content in.
        totals (MediaSourceTotals):
        published_period_years (Union[Unset, List[int]]): The range of years this media source has been published for.
            Impresso may not have data for all this period. Is not defined if there is no information.
        available_dates_range (Union[Unset, List[datetime.datetime]]): The range of dates this media source has content
            items for. This represents the earliest and the latest dates of the contet items.  Is not defined if there are
            no content items for this source.
        properties (Union[Unset, List['MediaSourcePropertiesItem']]):
    """

    uid: str
    type: MediaSourceType
    name: str
    language_codes: List[str]
    totals: "MediaSourceTotals"
    published_period_years: Union[Unset, List[int]] = UNSET
    available_dates_range: Union[Unset, List[datetime.datetime]] = UNSET
    properties: Union[Unset, List["MediaSourcePropertiesItem"]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        uid = self.uid

        type = self.type.value

        name = self.name

        language_codes = self.language_codes

        totals = self.totals.to_dict()

        published_period_years: Union[Unset, List[int]] = UNSET
        if not isinstance(self.published_period_years, Unset):
            published_period_years = self.published_period_years

        available_dates_range: Union[Unset, List[str]] = UNSET
        if not isinstance(self.available_dates_range, Unset):
            available_dates_range = []
            for available_dates_range_item_data in self.available_dates_range:
                available_dates_range_item = available_dates_range_item_data.isoformat()
                available_dates_range.append(available_dates_range_item)

        properties: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.properties, Unset):
            properties = []
            for properties_item_data in self.properties:
                properties_item = properties_item_data.to_dict()
                properties.append(properties_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "uid": uid,
                "type": type,
                "name": name,
                "languageCodes": language_codes,
                "totals": totals,
            }
        )
        if published_period_years is not UNSET:
            field_dict["publishedPeriodYears"] = published_period_years
        if available_dates_range is not UNSET:
            field_dict["availableDatesRange"] = available_dates_range
        if properties is not UNSET:
            field_dict["properties"] = properties

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.media_source_properties_item import MediaSourcePropertiesItem
        from ..models.media_source_totals import MediaSourceTotals

        d = src_dict.copy()
        uid = d.pop("uid")

        type = MediaSourceType(d.pop("type"))

        name = d.pop("name")

        language_codes = cast(List[str], d.pop("languageCodes"))

        totals = MediaSourceTotals.from_dict(d.pop("totals"))

        published_period_years = cast(List[int], d.pop("publishedPeriodYears", UNSET))

        available_dates_range = []
        _available_dates_range = d.pop("availableDatesRange", UNSET)
        for available_dates_range_item_data in _available_dates_range or []:
            available_dates_range_item = isoparse(available_dates_range_item_data)

            available_dates_range.append(available_dates_range_item)

        properties = []
        _properties = d.pop("properties", UNSET)
        for properties_item_data in _properties or []:
            properties_item = MediaSourcePropertiesItem.from_dict(properties_item_data)

            properties.append(properties_item)

        media_source = cls(
            uid=uid,
            type=type,
            name=name,
            language_codes=language_codes,
            totals=totals,
            published_period_years=published_period_years,
            available_dates_range=available_dates_range,
            properties=properties,
        )

        return media_source



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.authentication_create_request_strategy import AuthenticationCreateRequestStrategy
from ..types import UNSET, Unset

T = TypeVar("T", bound="AuthenticationCreateRequest")


@_attrs_define
class AuthenticationCreateRequest:
    """Request body for the authentication endpoint

    Attributes:
        strategy (AuthenticationCreateRequestStrategy):
        email (Union[Unset, str]):
        password (Union[Unset, str]):
        access_token (Union[Unset, str]):
    """

    strategy: AuthenticationCreateRequestStrategy
    email: Union[Unset, str] = UNSET
    password: Union[Unset, str] = UNSET
    access_token: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        strategy = self.strategy.value

        email = self.email

        password = self.password

        access_token = self.access_token

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "strategy": strategy,
            }
        )
        if email is not UNSET:
            field_dict["email"] = email
        if password is not UNSET:
            field_dict["password"] = password
        if access_token is not UNSET:
            field_dict["accessToken"] = access_token

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        strategy = AuthenticationCreateRequestStrategy(d.pop("strategy"))

        email = d.pop("email", UNSET)

        password = d.pop("password", UNSET)

        access_token = d.pop("accessToken", UNSET)

        authentication_create_request = cls(
            strategy=strategy,
            email=email,
            password=password,
            access_token=access_token,
        )

        authentication_create_request.additional_properties = d
        return authentication_create_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Callable, Iterator

from pandas import DataFrame, json_normalize
from pydantic import BaseModel

from impresso.api_client.api.content_items import get_content_item
from impresso.api_models import ContentItem, BaseFind
from impresso.data_container import DataContainer, iterate_pages
from impresso.resources.base import Resource
from impresso.util.error import raise_for_error


class ContentItemsResponseSchema(BaseFind):
    """Schema for the content items response."""

    data: list[ContentItem]


class ContentItemsDataContainer(DataContainer):
    """Response of a find content items call, supports pagination."""

    def __init__(
        self,
        data: BaseModel,
        pydantic_model: type[ContentItemsResponseSchema],
        fetch_method: Callable[..., "ContentItemsDataContainer"],
        fetch_method_args: dict[str, Any],
        web_app_search_result_url: str | None = None,
    ):
        super().__init__(data, pydantic_model, web_app_search_result_url)
        self._fetch_method = fetch_method
        self._fetch_method_args = fetch_method_args

    @property
    def df(self) -> DataFrame:
        """Return the data as a pandas dataframe."""
        return json_normalize(self._data.to_dict()["data"]).set_index("uid")

    def pages(self) -> Iterator["ContentItemsDataContainer"]:
        """Iterate over all pages of results."""
        yield self
        yield from iterate_pages(
            self._fetch_method,
            self._fetch_method_args,
            self.offset,
            self.limit,
            self.total,
        )


class ContentItemDataContainer(DataContainer):
    """Response of a get content item call."""

    @property
    def raw(self) -> dict[str, Any]:
        """Return the data as a python dictionary."""
        return self._data.to_dict()

    @property
    def pydantic(self) -> ContentItem:
        """Return the data as a pydantic model."""
        return self._pydantic_model.model_validate(self.raw)

    @property
    def df(self) -> DataFrame:
        """Return the data as a pandas dataframe."""
        return json_normalize([self.raw])

    @property
    def size(self) -> int:
        """Current page size."""
        data = self._data.to_dict()
        if len(data):
            return 1
        return 0

    @property
    def total(self) -> int:
        """Total number of results."""
        return self.size


class ContentItemsResource(Resource):
    """Get content items from the impresso database.

    Examples:
        Get a specific content item by its ID:
        >>> item_id = "some-item-id"  # Replace with a real ID
        >>> item = content_items.get(item_id)  # doctest: +SKIP
        >>> print(item.df)  # doctest: +SKIP

        Get a content item with embeddings:
        >>> item = content_items.get(item_id, include_embeddings=True)  # doctest: +SKIP
        >>> print(item.raw.get("embeddings"))  # doctest: +SKIP

        Get only the embeddings of a content item:
        >>> embeddings = content_items.get_embeddings(item_id)  # doctest: +SKIP
        >>> print(embeddings)  # doctest: +SKIP
    """

    name = "content_items"

    def get(
        self, id: str, include_embeddings: bool = False
    ) -> ContentItemDataContainer:
        """
        Get a content item by its id.

        Args:
            id: The id of the content item.
            include_embeddings: Whether to include embeddings in the response.

        Returns:
            ContentItemDataContainer: The content item data container.
        """
        result = get_content_item.sync(
            client=self._api_client,
            id=id,
            include_embeddings=include_embeddings,
        )
        raise_for_error(result)

        id_parts = id.split("-")
        issue_id = "-".join(id_parts[:-1])
        article_id = id_parts[-1]

        return ContentItemDataContainer(
            result,
            ContentItem,
            f"{self._get_web_app_base_url()}/issue/{issue_id}/view?articleId={article_id}",
        )

    def get_embeddings(self, id: str) -> list[str]:
        """
        Get the embeddings of a content item by its id.

        Args:
            id: The id of the content item.

        Returns:
            list[str]: The embeddings of the content item if present (every embedding is returned
                in the canonical form: <model>:<base64_embedding>).
        """
        item = self.get(id, include_embeddings=True)
        return item.raw.get("embeddings", []) if item else []
